from enum import Enum


class SessionThresholdCheckDtoStatus(str, Enum):
    APPROVED = "Approved"
    PENDING = "Pending"
    REJECTED = "Rejected"
    SKIPPED = "Skipped"
    SUSPICIOUS = "Suspicious"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class MessageLogDtoDirection(str, Enum):
    CHARGERTOCPO = "ChargerToCpo"
    CPOTOCHARGER = "CpoToCharger"
    UNKNOWN = "Unknown"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
import datetime
from typing import Union


T = TypeVar("T", bound="ReimbursementBankDetailsDto")


@_attrs_define
class ReimbursementBankDetailsDto:
    """
    Attributes:
        bankaccount (Union[Unset, str]):
        date_created (Union[Unset, datetime.datetime]):
        valid_from (Union[Unset, datetime.datetime]):
    """

    bankaccount: Union[Unset, str] = UNSET
    date_created: Union[Unset, datetime.datetime] = UNSET
    valid_from: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        bankaccount = self.bankaccount

        date_created: Union[Unset, str] = UNSET
        if not isinstance(self.date_created, Unset):
            date_created = self.date_created.isoformat()

        valid_from: Union[Unset, str] = UNSET
        if not isinstance(self.valid_from, Unset):
            valid_from = self.valid_from.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if bankaccount is not UNSET:
            field_dict["bankaccount"] = bankaccount
        if date_created is not UNSET:
            field_dict["dateCreated"] = date_created
        if valid_from is not UNSET:
            field_dict["validFrom"] = valid_from

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        bankaccount = d.pop("bankaccount", UNSET)

        _date_created = d.pop("dateCreated", UNSET)
        date_created: Union[Unset, datetime.datetime]
        if isinstance(_date_created, Unset) or _date_created is None:
            date_created = UNSET
        else:
            date_created = isoparse(_date_created)

        _valid_from = d.pop("validFrom", UNSET)
        valid_from: Union[Unset, datetime.datetime]
        if isinstance(_valid_from, Unset) or _valid_from is None:
            valid_from = UNSET
        else:
            valid_from = isoparse(_valid_from)

        reimbursement_bank_details_dto = cls(
            bankaccount=bankaccount,
            date_created=date_created,
            valid_from=valid_from,
        )

        reimbursement_bank_details_dto.additional_properties = d
        return reimbursement_bank_details_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class AuthorizationAssertionDtoStatus(str, Enum):
    INVALID = "Invalid"
    PENDING = "Pending"
    SKIPPED = "Skipped"
    TIMEOUT = "Timeout"
    VALID = "Valid"
    VERIFIED = "Verified"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import Union

if TYPE_CHECKING:
    from ..models.local_token_group_token_post_dto import LocalTokenGroupTokenPostDto


T = TypeVar("T", bound="LocalTokenGroupPostDto")


@_attrs_define
class LocalTokenGroupPostDto:
    """
    Attributes:
        oucode (Union[Unset, str]):
        token_group_name (Union[Unset, str]):
        target_ou_codes (Union[Unset, List[str]]):
        override_tariff_id (Union[Unset, str]):
        tokens (Union[Unset, List['LocalTokenGroupTokenPostDto']]):
        target_chargepoint_ids (Union[Unset, List[str]]):
    """

    oucode: Union[Unset, str] = UNSET
    token_group_name: Union[Unset, str] = UNSET
    target_ou_codes: Union[Unset, List[str]] = UNSET
    override_tariff_id: Union[Unset, str] = UNSET
    tokens: Union[Unset, List["LocalTokenGroupTokenPostDto"]] = UNSET
    target_chargepoint_ids: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        oucode = self.oucode

        token_group_name = self.token_group_name

        target_ou_codes: Union[Unset, List[str]] = UNSET
        if not isinstance(self.target_ou_codes, Unset):
            target_ou_codes = self.target_ou_codes

        override_tariff_id = self.override_tariff_id

        tokens: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.tokens, Unset):
            tokens = []
            for tokens_item_data in self.tokens:
                tokens_item = tokens_item_data.to_dict()
                tokens.append(tokens_item)

        target_chargepoint_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.target_chargepoint_ids, Unset):
            target_chargepoint_ids = self.target_chargepoint_ids

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if oucode is not UNSET:
            field_dict["oucode"] = oucode
        if token_group_name is not UNSET:
            field_dict["tokenGroupName"] = token_group_name
        if target_ou_codes is not UNSET:
            field_dict["targetOUCodes"] = target_ou_codes
        if override_tariff_id is not UNSET:
            field_dict["overrideTariffId"] = override_tariff_id
        if tokens is not UNSET:
            field_dict["tokens"] = tokens
        if target_chargepoint_ids is not UNSET:
            field_dict["targetChargepointIds"] = target_chargepoint_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.local_token_group_token_post_dto import (
            LocalTokenGroupTokenPostDto,
        )

        d = src_dict.copy()
        oucode = d.pop("oucode", UNSET)

        token_group_name = d.pop("tokenGroupName", UNSET)

        target_ou_codes = cast(List[str], d.pop("targetOUCodes", UNSET))

        override_tariff_id = d.pop("overrideTariffId", UNSET)

        tokens = []
        _tokens = d.pop("tokens", UNSET)
        for tokens_item_data in _tokens or []:
            tokens_item = LocalTokenGroupTokenPostDto.from_dict(tokens_item_data)

            tokens.append(tokens_item)

        target_chargepoint_ids = cast(List[str], d.pop("targetChargepointIds", UNSET))

        local_token_group_post_dto = cls(
            oucode=oucode,
            token_group_name=token_group_name,
            target_ou_codes=target_ou_codes,
            override_tariff_id=override_tariff_id,
            tokens=tokens,
            target_chargepoint_ids=target_chargepoint_ids,
        )

        local_token_group_post_dto.additional_properties = d
        return local_token_group_post_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.organization_unit_financial_details_dto import (
        OrganizationUnitFinancialDetailsDto,
    )


T = TypeVar("T", bound="OrganizationUnitPostDto")


@_attrs_define
class OrganizationUnitPostDto:
    """
    Attributes:
        parent_id (str): The id of parent ou.
        name (Union[Unset, str]): The name of the ou.
        external_reference (Union[Unset, str]): This property can be used to link this OU to another system.
        grid_owner_reference (Union[Unset, str]): This property can be used to link this OU to a grid owner.
        tenant_reference (Union[Unset, str]): This property can be used to link this OU to a tenant.
        customer_reference (Union[Unset, str]): This property can be used to link this OU to a customer.
        address (Union[Unset, str]):
        state (Union[Unset, str]):
        country (Union[Unset, str]):
        city (Union[Unset, str]):
        house_number (Union[Unset, str]):
        postal_code (Union[Unset, str]):
        hotline_phone_number (Union[Unset, str]):
        company_email (Union[Unset, str]):
        primary_contactperson (Union[Unset, str]):
        primary_contactperson_email (Union[Unset, str]):
        msp_ou_id (Union[Unset, str]): The ou id used for the "Msp Integration" feature.
        msp_ou_name (Union[Unset, str]): The ou name used for the "Msp Integration" feature.
        msp_ou_code (Union[Unset, str]): The ou code used for the "Msp Integration" feature.
        msp_external_id (Union[Unset, str]): The externalId from the "Msp Integration" feature.
        financial_details (Union[Unset, OrganizationUnitFinancialDetailsDto]):
    """

    parent_id: str
    name: Union[Unset, str] = UNSET
    external_reference: Union[Unset, str] = UNSET
    grid_owner_reference: Union[Unset, str] = UNSET
    tenant_reference: Union[Unset, str] = UNSET
    customer_reference: Union[Unset, str] = UNSET
    address: Union[Unset, str] = UNSET
    state: Union[Unset, str] = UNSET
    country: Union[Unset, str] = UNSET
    city: Union[Unset, str] = UNSET
    house_number: Union[Unset, str] = UNSET
    postal_code: Union[Unset, str] = UNSET
    hotline_phone_number: Union[Unset, str] = UNSET
    company_email: Union[Unset, str] = UNSET
    primary_contactperson: Union[Unset, str] = UNSET
    primary_contactperson_email: Union[Unset, str] = UNSET
    msp_ou_id: Union[Unset, str] = UNSET
    msp_ou_name: Union[Unset, str] = UNSET
    msp_ou_code: Union[Unset, str] = UNSET
    msp_external_id: Union[Unset, str] = UNSET
    financial_details: Union[Unset, "OrganizationUnitFinancialDetailsDto"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        parent_id = self.parent_id

        name = self.name

        external_reference = self.external_reference

        grid_owner_reference = self.grid_owner_reference

        tenant_reference = self.tenant_reference

        customer_reference = self.customer_reference

        address = self.address

        state = self.state

        country = self.country

        city = self.city

        house_number = self.house_number

        postal_code = self.postal_code

        hotline_phone_number = self.hotline_phone_number

        company_email = self.company_email

        primary_contactperson = self.primary_contactperson

        primary_contactperson_email = self.primary_contactperson_email

        msp_ou_id = self.msp_ou_id

        msp_ou_name = self.msp_ou_name

        msp_ou_code = self.msp_ou_code

        msp_external_id = self.msp_external_id

        financial_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.financial_details, Unset):
            financial_details = self.financial_details.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "parentId": parent_id,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name
        if external_reference is not UNSET:
            field_dict["externalReference"] = external_reference
        if grid_owner_reference is not UNSET:
            field_dict["gridOwnerReference"] = grid_owner_reference
        if tenant_reference is not UNSET:
            field_dict["tenantReference"] = tenant_reference
        if customer_reference is not UNSET:
            field_dict["customerReference"] = customer_reference
        if address is not UNSET:
            field_dict["address"] = address
        if state is not UNSET:
            field_dict["state"] = state
        if country is not UNSET:
            field_dict["country"] = country
        if city is not UNSET:
            field_dict["city"] = city
        if house_number is not UNSET:
            field_dict["houseNumber"] = house_number
        if postal_code is not UNSET:
            field_dict["postalCode"] = postal_code
        if hotline_phone_number is not UNSET:
            field_dict["hotlinePhoneNumber"] = hotline_phone_number
        if company_email is not UNSET:
            field_dict["companyEmail"] = company_email
        if primary_contactperson is not UNSET:
            field_dict["primaryContactperson"] = primary_contactperson
        if primary_contactperson_email is not UNSET:
            field_dict["primaryContactpersonEmail"] = primary_contactperson_email
        if msp_ou_id is not UNSET:
            field_dict["mspOuId"] = msp_ou_id
        if msp_ou_name is not UNSET:
            field_dict["mspOuName"] = msp_ou_name
        if msp_ou_code is not UNSET:
            field_dict["mspOuCode"] = msp_ou_code
        if msp_external_id is not UNSET:
            field_dict["mspExternalId"] = msp_external_id
        if financial_details is not UNSET:
            field_dict["financialDetails"] = financial_details

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.organization_unit_financial_details_dto import (
            OrganizationUnitFinancialDetailsDto,
        )

        d = src_dict.copy()
        parent_id = d.pop("parentId")

        name = d.pop("name", UNSET)

        external_reference = d.pop("externalReference", UNSET)

        grid_owner_reference = d.pop("gridOwnerReference", UNSET)

        tenant_reference = d.pop("tenantReference", UNSET)

        customer_reference = d.pop("customerReference", UNSET)

        address = d.pop("address", UNSET)

        state = d.pop("state", UNSET)

        country = d.pop("country", UNSET)

        city = d.pop("city", UNSET)

        house_number = d.pop("houseNumber", UNSET)

        postal_code = d.pop("postalCode", UNSET)

        hotline_phone_number = d.pop("hotlinePhoneNumber", UNSET)

        company_email = d.pop("companyEmail", UNSET)

        primary_contactperson = d.pop("primaryContactperson", UNSET)

        primary_contactperson_email = d.pop("primaryContactpersonEmail", UNSET)

        msp_ou_id = d.pop("mspOuId", UNSET)

        msp_ou_name = d.pop("mspOuName", UNSET)

        msp_ou_code = d.pop("mspOuCode", UNSET)

        msp_external_id = d.pop("mspExternalId", UNSET)

        _financial_details = d.pop("financialDetails", UNSET)
        financial_details: Union[Unset, OrganizationUnitFinancialDetailsDto]
        if isinstance(_financial_details, Unset) or _financial_details is None:
            financial_details = UNSET
        else:
            financial_details = OrganizationUnitFinancialDetailsDto.from_dict(
                _financial_details
            )

        organization_unit_post_dto = cls(
            parent_id=parent_id,
            name=name,
            external_reference=external_reference,
            grid_owner_reference=grid_owner_reference,
            tenant_reference=tenant_reference,
            customer_reference=customer_reference,
            address=address,
            state=state,
            country=country,
            city=city,
            house_number=house_number,
            postal_code=postal_code,
            hotline_phone_number=hotline_phone_number,
            company_email=company_email,
            primary_contactperson=primary_contactperson,
            primary_contactperson_email=primary_contactperson_email,
            msp_ou_id=msp_ou_id,
            msp_ou_name=msp_ou_name,
            msp_ou_code=msp_ou_code,
            msp_external_id=msp_external_id,
            financial_details=financial_details,
        )

        organization_unit_post_dto.additional_properties = d
        return organization_unit_post_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .additional_geo_location_dto import AdditionalGeoLocationDto
from .authorization_assertion_dto import AuthorizationAssertionDto
from .authorization_assertion_dto_auth_scenario_type import (
    AuthorizationAssertionDtoAuthScenarioType,
)
from .authorization_assertion_dto_status import AuthorizationAssertionDtoStatus
from .authorization_charger_context_dto import AuthorizationChargerContextDto
from .authorization_context_details_dto import AuthorizationContextDetailsDto
from .authorization_data import AuthorizationData
from .authorization_result_dto import AuthorizationResultDto
from .authorization_result_dto_reason import AuthorizationResultDtoReason
from .authorization_result_dto_status import AuthorizationResultDtoStatus
from .authorization_tenant_context_dto import AuthorizationTenantContextDto
from .business_details_dto import BusinessDetailsDto
from .cancel_reservation_request import CancelReservationRequest
from .cdr_dto import CdrDto
from .cdr_dto_approval_status import CdrDtoApprovalStatus
from .cdr_dto_financial_type import CdrDtoFinancialType
from .cdr_geo_location_dto import CdrGeoLocationDto
from .cdr_location_dto import CdrLocationDto
from .cdr_location_dto_power_type import CdrLocationDtoPowerType
from .cdr_patch_dto import CdrPatchDto
from .cdr_patch_dto_approval_status import CdrPatchDtoApprovalStatus
from .cdr_started_by_info_dto import CdrStartedByInfoDto
from .cdr_started_by_info_dto_authorization_state import (
    CdrStartedByInfoDtoAuthorizationState,
)
from .cdr_started_by_info_dto_roaming_platform_type import (
    CdrStartedByInfoDtoRoamingPlatformType,
)
from .cdr_started_by_token_dto import CdrStartedByTokenDto
from .cdr_started_by_token_dto_auth_method import CdrStartedByTokenDtoAuthMethod
from .cdr_started_by_token_dto_token_type import CdrStartedByTokenDtoTokenType
from .change_availability_request import ChangeAvailabilityRequest
from .change_availability_request_type import ChangeAvailabilityRequestType
from .change_configuration_request import ChangeConfigurationRequest
from .charge_point_authorize_get_dto import ChargePointAuthorizeGetDto
from .charge_point_authorize_get_dto_authorization_request_type import (
    ChargePointAuthorizeGetDtoAuthorizationRequestType,
)
from .charge_point_authorize_post_dto import ChargePointAuthorizePostDto
from .chargepoint_configuration_items_dto import ChargepointConfigurationItemsDto
from .chargepoint_connector_dto import ChargepointConnectorDto
from .chargepoint_connector_dto_format import ChargepointConnectorDtoFormat
from .chargepoint_connector_dto_operational_status import (
    ChargepointConnectorDtoOperationalStatus,
)
from .chargepoint_connector_dto_power_type import ChargepointConnectorDtoPowerType
from .chargepoint_connector_dto_standard import ChargepointConnectorDtoStandard
from .chargepoint_dto import ChargepointDto
from .chargepoint_dto_connectivity_status import ChargepointDtoConnectivityStatus
from .chargepoint_evse_dto import ChargepointEVSEDto
from .chargepoint_put_dto import ChargepointPutDto
from .chargepoint_status_dto import ChargepointStatusDto
from .chargepoint_status_dto_connectivity_status import (
    ChargepointStatusDtoConnectivityStatus,
)
from .charging_meter_value_dto import ChargingMeterValueDto
from .charging_meter_value_dto_measurand import ChargingMeterValueDtoMeasurand
from .charging_meter_value_dto_unit import ChargingMeterValueDtoUnit
from .charging_period_dto import ChargingPeriodDto
from .charging_profile import ChargingProfile
from .charging_profile_charging_profile_kind import ChargingProfileChargingProfileKind
from .charging_profile_charging_profile_purpose import (
    ChargingProfileChargingProfilePurpose,
)
from .charging_profile_recurrency_kind import ChargingProfileRecurrencyKind
from .charging_schedule import ChargingSchedule
from .charging_schedule_charging_rate_unit import ChargingScheduleChargingRateUnit
from .charging_schedule_period import ChargingSchedulePeriod
from .clear_cache_request import ClearCacheRequest
from .clear_charging_profile_request import ClearChargingProfileRequest
from .clear_charging_profile_request_charging_profile_purpose import (
    ClearChargingProfileRequestChargingProfilePurpose,
)
from .connector_dto import ConnectorDto
from .connector_dto_format import ConnectorDtoFormat
from .connector_dto_power_type import ConnectorDtoPowerType
from .connector_dto_standard import ConnectorDtoStandard
from .connector_operational_status_dto import ConnectorOperationalStatusDto
from .connector_operational_status_dto_operational_status import (
    ConnectorOperationalStatusDtoOperationalStatus,
)
from .cs_charging_profiles import CsChargingProfiles
from .cs_charging_profiles_charging_profile_kind import (
    CsChargingProfilesChargingProfileKind,
)
from .cs_charging_profiles_charging_profile_purpose import (
    CsChargingProfilesChargingProfilePurpose,
)
from .cs_charging_profiles_recurrency_kind import CsChargingProfilesRecurrencyKind
from .data_transfer_request import DataTransferRequest
from .display_text_dto import DisplayTextDto
from .energy_mix_dto import EnergyMixDto
from .energy_source_dto import EnergySourceDto
from .energy_source_dto_source import EnergySourceDtoSource
from .entity_tag_header_value import EntityTagHeaderValue
from .environmental_impact_dto import EnvironmentalImpactDto
from .environmental_impact_dto_category import EnvironmentalImpactDtoCategory
from .exceptional_period_dto import ExceptionalPeriodDto
from .file_content_result import FileContentResult
from .geo_location_dto import GeoLocationDto
from .get_all_cdrs_order_by import GetAllCdrsOrderBy
from .get_all_chargepoints_accesstype import GetAllChargepointsAccesstype
from .get_all_chargepoints_chargerpowertype import GetAllChargepointsChargerpowertype
from .get_all_chargepoints_operationalstatus import GetAllChargepointsOperationalstatus
from .get_all_chargepoints_order_by import GetAllChargepointsOrderBy
from .get_all_locations_accesstype import GetAllLocationsAccesstype
from .get_all_locations_chargerpowertype import GetAllLocationsChargerpowertype
from .get_all_locations_order_by import GetAllLocationsOrderBy
from .get_all_reimbursementcdrs_order_by import GetAllReimbursementcdrsOrderBy
from .get_all_sessions_order_by import GetAllSessionsOrderBy
from .get_all_tariffdistributions_order_by import GetAllTariffdistributionsOrderBy
from .get_all_tariffs_order_by import GetAllTariffsOrderBy
from .get_all_webhooks_order_by import GetAllWebhooksOrderBy
from .get_composite_schedule_request import GetCompositeScheduleRequest
from .get_composite_schedule_request_charging_rate_unit import (
    GetCompositeScheduleRequestChargingRateUnit,
)
from .get_configuration_request import GetConfigurationRequest
from .get_diagnostics_request import GetDiagnosticsRequest
from .get_local_list_version_request import GetLocalListVersionRequest
from .hours_dto import HoursDto
from .id_tag_info import IdTagInfo
from .id_tag_info_status import IdTagInfoStatus
from .image_dto import ImageDto
from .image_dto_category import ImageDtoCategory
from .interchange_format_cdr import InterchangeFormatCdr
from .local_token_group_get_dto import LocalTokenGroupGetDto
from .local_token_group_post_dto import LocalTokenGroupPostDto
from .local_token_group_put_dto import LocalTokenGroupPutDto
from .local_token_group_token_get_dto import LocalTokenGroupTokenGetDto
from .local_token_group_token_post_dto import LocalTokenGroupTokenPostDto
from .local_token_group_token_put_dto import LocalTokenGroupTokenPutDto
from .location_charge_point_dto import LocationChargePointDto
from .location_dto import LocationDto
from .location_dto_facilities_item import LocationDtoFacilitiesItem
from .location_dto_parking_type import LocationDtoParkingType
from .location_evse_dto import LocationEVSEDto
from .location_evse_dto_capabilities_item import LocationEVSEDtoCapabilitiesItem
from .location_evse_dto_parking_restrictions_item import (
    LocationEVSEDtoParkingRestrictionsItem,
)
from .location_evse_dto_status import LocationEVSEDtoStatus
from .location_post_dto import LocationPostDto
from .location_post_dto_facilities_item import LocationPostDtoFacilitiesItem
from .location_post_dto_parking_type import LocationPostDtoParkingType
from .location_put_dto import LocationPutDto
from .location_put_dto_facilities_item import LocationPutDtoFacilitiesItem
from .location_put_dto_parking_type import LocationPutDtoParkingType
from .location_tariff_distribution_dto import LocationTariffDistributionDto
from .longship_error import LongshipError
from .longship_error_detail import LongshipErrorDetail
from .message_log_dto import MessageLogDto
from .message_log_dto_direction import MessageLogDtoDirection
from .message_log_dto_ocpp_message_type import MessageLogDtoOcppMessageType
from .message_log_dto_wamp_message_type import MessageLogDtoWampMessageType
from .organization_unit_financial_details_dto import OrganizationUnitFinancialDetailsDto
from .organization_unit_get_dto import OrganizationUnitGetDto
from .organization_unit_post_dto import OrganizationUnitPostDto
from .organization_unit_put_dto import OrganizationUnitPutDto
from .price_info_dto import PriceInfoDto
from .private_emp_tariff_dto import PrivateEmpTariffDto
from .private_emp_tariff_dto_power_type import PrivateEmpTariffDtoPowerType
from .publish_token_type_dto import PublishTokenTypeDto
from .publish_token_type_dto_type import PublishTokenTypeDtoType
from .regular_hours_dto import RegularHoursDto
from .reimburse_info_dto import ReimburseInfoDto
from .reimburse_info_dto_type import ReimburseInfoDtoType
from .reimburse_started_by_info_dto import ReimburseStartedByInfoDto
from .reimburse_started_by_info_dto_authorization_state import (
    ReimburseStartedByInfoDtoAuthorizationState,
)
from .reimburse_started_by_token_dto import ReimburseStartedByTokenDto
from .reimburse_started_by_token_dto_auth_method import (
    ReimburseStartedByTokenDtoAuthMethod,
)
from .reimburse_started_by_token_dto_token_type import (
    ReimburseStartedByTokenDtoTokenType,
)
from .reimbursement_bank_details_dto import ReimbursementBankDetailsDto
from .reimbursement_cdr_dto import ReimbursementCdrDto
from .reimbursement_cdr_geo_location_dto import ReimbursementCdrGeoLocationDto
from .reimbursement_cdr_location_dto import ReimbursementCdrLocationDto
from .reimbursement_cdr_location_dto_power_type import (
    ReimbursementCdrLocationDtoPowerType,
)
from .reimbursement_customer_share_dto import ReimbursementCustomerShareDto
from .reimbursement_price_dto import ReimbursementPriceDto
from .reimbursement_tariff_dto import ReimbursementTariffDto
from .reimbursement_tariff_dto_status import ReimbursementTariffDtoStatus
from .remote_start_transaction_request import RemoteStartTransactionRequest
from .remote_stop_transaction_request import RemoteStopTransactionRequest
from .reserve_now_request import ReserveNowRequest
from .reset_request import ResetRequest
from .reset_request_type import ResetRequestType
from .send_local_list_request import SendLocalListRequest
from .send_local_list_request_update_type import SendLocalListRequestUpdateType
from .session_dto import SessionDto
from .session_dto_approval_status import SessionDtoApprovalStatus
from .session_dto_review_scenario_type import SessionDtoReviewScenarioType
from .session_dto_status import SessionDtoStatus
from .session_geo_location_dto import SessionGeoLocationDto
from .session_location_dto import SessionLocationDto
from .session_location_dto_power_type import SessionLocationDtoPowerType
from .session_threshold_check_dto import SessionThresholdCheckDto
from .session_threshold_check_dto_status import SessionThresholdCheckDtoStatus
from .session_threshold_check_dto_threshold_hit_outcome import (
    SessionThresholdCheckDtoThresholdHitOutcome,
)
from .session_threshold_value_dto_decimal import SessionThresholdValueDtoDecimal
from .session_threshold_value_dto_decimal_status import (
    SessionThresholdValueDtoDecimalStatus,
)
from .session_threshold_value_dto_decimal_threshold_hit_outcome import (
    SessionThresholdValueDtoDecimalThresholdHitOutcome,
)
from .session_threshold_value_dto_int_32 import SessionThresholdValueDtoInt32
from .session_threshold_value_dto_int_32_status import (
    SessionThresholdValueDtoInt32Status,
)
from .session_threshold_value_dto_int_32_threshold_hit_outcome import (
    SessionThresholdValueDtoInt32ThresholdHitOutcome,
)
from .session_thresholds_dto import SessionThresholdsDto
from .session_thresholds_dto_thresholds_hit_item import (
    SessionThresholdsDtoThresholdsHitItem,
)
from .set_charging_profile_request import SetChargingProfileRequest
from .started_by_info_dto import StartedByInfoDto
from .started_by_info_dto_authorization_state import StartedByInfoDtoAuthorizationState
from .started_by_info_dto_roaming_platform_type import (
    StartedByInfoDtoRoamingPlatformType,
)
from .started_by_token_dto import StartedByTokenDto
from .started_by_token_dto_auth_method import StartedByTokenDtoAuthMethod
from .started_by_token_dto_token_type import StartedByTokenDtoTokenType
from .status_schedule_dto import StatusScheduleDto
from .status_schedule_dto_status import StatusScheduleDtoStatus
from .string_segment import StringSegment
from .tariff_assertion_dto import TariffAssertionDto
from .tariff_assertion_dto_tariff_type import TariffAssertionDtoTariffType
from .tariff_distribution_get_dto import TariffDistributionGetDto
from .tariff_distribution_history_dto import TariffDistributionHistoryDto
from .tariff_distribution_post_dto import TariffDistributionPostDto
from .tariff_distribution_put_dto import TariffDistributionPutDto
from .tariff_dto import TariffDto
from .tariff_dto_tariff_type import TariffDtoTariffType
from .tariff_dto_usage_type import TariffDtoUsageType
from .tariff_info_dto import TariffInfoDto
from .tariff_post_dto import TariffPostDto
from .tariff_post_dto_usage_type import TariffPostDtoUsageType
from .tariff_price_dto import TariffPriceDto
from .tariff_price_dto_approval_status import TariffPriceDtoApprovalStatus
from .tariff_put_dto import TariffPutDto
from .tariff_restriction import TariffRestriction
from .tariff_restriction_day_of_week import TariffRestrictionDayOfWeek
from .token_info_dto import TokenInfoDto
from .token_info_dto_token_type import TokenInfoDtoTokenType
from .trigger_message_request import TriggerMessageRequest
from .trigger_message_request_requested_message import (
    TriggerMessageRequestRequestedMessage,
)
from .unlock_connector_request import UnlockConnectorRequest
from .update_firmware_request import UpdateFirmwareRequest
from .webhook_get_dto import WebhookGetDto
from .webhook_get_dto_event_types_item import WebhookGetDtoEventTypesItem
from .webhook_header_dto import WebhookHeaderDto
from .webhook_post_dto import WebhookPostDto
from .webhook_post_dto_event_types_item import WebhookPostDtoEventTypesItem
from .webhook_put_dto import WebhookPutDto
from .webhook_put_dto_event_types_item import WebhookPutDtoEventTypesItem
from .webhook_summary_get_dto import WebhookSummaryGetDto
from .webhook_summary_get_dto_event_types_item import WebhookSummaryGetDtoEventTypesItem

__all__ = (
    "AdditionalGeoLocationDto",
    "AuthorizationAssertionDto",
    "AuthorizationAssertionDtoAuthScenarioType",
    "AuthorizationAssertionDtoStatus",
    "AuthorizationChargerContextDto",
    "AuthorizationContextDetailsDto",
    "AuthorizationData",
    "AuthorizationResultDto",
    "AuthorizationResultDtoReason",
    "AuthorizationResultDtoStatus",
    "AuthorizationTenantContextDto",
    "BusinessDetailsDto",
    "CancelReservationRequest",
    "CdrDto",
    "CdrDtoApprovalStatus",
    "CdrDtoFinancialType",
    "CdrGeoLocationDto",
    "CdrLocationDto",
    "CdrLocationDtoPowerType",
    "CdrPatchDto",
    "CdrPatchDtoApprovalStatus",
    "CdrStartedByInfoDto",
    "CdrStartedByInfoDtoAuthorizationState",
    "CdrStartedByInfoDtoRoamingPlatformType",
    "CdrStartedByTokenDto",
    "CdrStartedByTokenDtoAuthMethod",
    "CdrStartedByTokenDtoTokenType",
    "ChangeAvailabilityRequest",
    "ChangeAvailabilityRequestType",
    "ChangeConfigurationRequest",
    "ChargePointAuthorizeGetDto",
    "ChargePointAuthorizeGetDtoAuthorizationRequestType",
    "ChargePointAuthorizePostDto",
    "ChargepointConfigurationItemsDto",
    "ChargepointConnectorDto",
    "ChargepointConnectorDtoFormat",
    "ChargepointConnectorDtoOperationalStatus",
    "ChargepointConnectorDtoPowerType",
    "ChargepointConnectorDtoStandard",
    "ChargepointDto",
    "ChargepointDtoConnectivityStatus",
    "ChargepointEVSEDto",
    "ChargepointPutDto",
    "ChargepointStatusDto",
    "ChargepointStatusDtoConnectivityStatus",
    "ChargingMeterValueDto",
    "ChargingMeterValueDtoMeasurand",
    "ChargingMeterValueDtoUnit",
    "ChargingPeriodDto",
    "ChargingProfile",
    "ChargingProfileChargingProfileKind",
    "ChargingProfileChargingProfilePurpose",
    "ChargingProfileRecurrencyKind",
    "ChargingSchedule",
    "ChargingScheduleChargingRateUnit",
    "ChargingSchedulePeriod",
    "ClearCacheRequest",
    "ClearChargingProfileRequest",
    "ClearChargingProfileRequestChargingProfilePurpose",
    "ConnectorDto",
    "ConnectorDtoFormat",
    "ConnectorDtoPowerType",
    "ConnectorDtoStandard",
    "ConnectorOperationalStatusDto",
    "ConnectorOperationalStatusDtoOperationalStatus",
    "CsChargingProfiles",
    "CsChargingProfilesChargingProfileKind",
    "CsChargingProfilesChargingProfilePurpose",
    "CsChargingProfilesRecurrencyKind",
    "DataTransferRequest",
    "DisplayTextDto",
    "EnergyMixDto",
    "EnergySourceDto",
    "EnergySourceDtoSource",
    "EntityTagHeaderValue",
    "EnvironmentalImpactDto",
    "EnvironmentalImpactDtoCategory",
    "ExceptionalPeriodDto",
    "FileContentResult",
    "GeoLocationDto",
    "GetAllCdrsOrderBy",
    "GetAllChargepointsAccesstype",
    "GetAllChargepointsChargerpowertype",
    "GetAllChargepointsOperationalstatus",
    "GetAllChargepointsOrderBy",
    "GetAllLocationsAccesstype",
    "GetAllLocationsChargerpowertype",
    "GetAllLocationsOrderBy",
    "GetAllReimbursementcdrsOrderBy",
    "GetAllSessionsOrderBy",
    "GetAllTariffdistributionsOrderBy",
    "GetAllTariffsOrderBy",
    "GetAllWebhooksOrderBy",
    "GetCompositeScheduleRequest",
    "GetCompositeScheduleRequestChargingRateUnit",
    "GetConfigurationRequest",
    "GetDiagnosticsRequest",
    "GetLocalListVersionRequest",
    "HoursDto",
    "IdTagInfo",
    "IdTagInfoStatus",
    "ImageDto",
    "ImageDtoCategory",
    "InterchangeFormatCdr",
    "LocalTokenGroupGetDto",
    "LocalTokenGroupPostDto",
    "LocalTokenGroupPutDto",
    "LocalTokenGroupTokenGetDto",
    "LocalTokenGroupTokenPostDto",
    "LocalTokenGroupTokenPutDto",
    "LocationChargePointDto",
    "LocationDto",
    "LocationDtoFacilitiesItem",
    "LocationDtoParkingType",
    "LocationEVSEDto",
    "LocationEVSEDtoCapabilitiesItem",
    "LocationEVSEDtoParkingRestrictionsItem",
    "LocationEVSEDtoStatus",
    "LocationPostDto",
    "LocationPostDtoFacilitiesItem",
    "LocationPostDtoParkingType",
    "LocationPutDto",
    "LocationPutDtoFacilitiesItem",
    "LocationPutDtoParkingType",
    "LocationTariffDistributionDto",
    "LongshipError",
    "LongshipErrorDetail",
    "MessageLogDto",
    "MessageLogDtoDirection",
    "MessageLogDtoOcppMessageType",
    "MessageLogDtoWampMessageType",
    "OrganizationUnitFinancialDetailsDto",
    "OrganizationUnitGetDto",
    "OrganizationUnitPostDto",
    "OrganizationUnitPutDto",
    "PriceInfoDto",
    "PrivateEmpTariffDto",
    "PrivateEmpTariffDtoPowerType",
    "PublishTokenTypeDto",
    "PublishTokenTypeDtoType",
    "RegularHoursDto",
    "ReimburseInfoDto",
    "ReimburseInfoDtoType",
    "ReimbursementBankDetailsDto",
    "ReimbursementCdrDto",
    "ReimbursementCdrGeoLocationDto",
    "ReimbursementCdrLocationDto",
    "ReimbursementCdrLocationDtoPowerType",
    "ReimbursementCustomerShareDto",
    "ReimbursementPriceDto",
    "ReimbursementTariffDto",
    "ReimbursementTariffDtoStatus",
    "ReimburseStartedByInfoDto",
    "ReimburseStartedByInfoDtoAuthorizationState",
    "ReimburseStartedByTokenDto",
    "ReimburseStartedByTokenDtoAuthMethod",
    "ReimburseStartedByTokenDtoTokenType",
    "RemoteStartTransactionRequest",
    "RemoteStopTransactionRequest",
    "ReserveNowRequest",
    "ResetRequest",
    "ResetRequestType",
    "SendLocalListRequest",
    "SendLocalListRequestUpdateType",
    "SessionDto",
    "SessionDtoApprovalStatus",
    "SessionDtoReviewScenarioType",
    "SessionDtoStatus",
    "SessionGeoLocationDto",
    "SessionLocationDto",
    "SessionLocationDtoPowerType",
    "SessionThresholdCheckDto",
    "SessionThresholdCheckDtoStatus",
    "SessionThresholdCheckDtoThresholdHitOutcome",
    "SessionThresholdsDto",
    "SessionThresholdsDtoThresholdsHitItem",
    "SessionThresholdValueDtoDecimal",
    "SessionThresholdValueDtoDecimalStatus",
    "SessionThresholdValueDtoDecimalThresholdHitOutcome",
    "SessionThresholdValueDtoInt32",
    "SessionThresholdValueDtoInt32Status",
    "SessionThresholdValueDtoInt32ThresholdHitOutcome",
    "SetChargingProfileRequest",
    "StartedByInfoDto",
    "StartedByInfoDtoAuthorizationState",
    "StartedByInfoDtoRoamingPlatformType",
    "StartedByTokenDto",
    "StartedByTokenDtoAuthMethod",
    "StartedByTokenDtoTokenType",
    "StatusScheduleDto",
    "StatusScheduleDtoStatus",
    "StringSegment",
    "TariffAssertionDto",
    "TariffAssertionDtoTariffType",
    "TariffDistributionGetDto",
    "TariffDistributionHistoryDto",
    "TariffDistributionPostDto",
    "TariffDistributionPutDto",
    "TariffDto",
    "TariffDtoTariffType",
    "TariffDtoUsageType",
    "TariffInfoDto",
    "TariffPostDto",
    "TariffPostDtoUsageType",
    "TariffPriceDto",
    "TariffPriceDtoApprovalStatus",
    "TariffPutDto",
    "TariffRestriction",
    "TariffRestrictionDayOfWeek",
    "TokenInfoDto",
    "TokenInfoDtoTokenType",
    "TriggerMessageRequest",
    "TriggerMessageRequestRequestedMessage",
    "UnlockConnectorRequest",
    "UpdateFirmwareRequest",
    "WebhookGetDto",
    "WebhookGetDtoEventTypesItem",
    "WebhookHeaderDto",
    "WebhookPostDto",
    "WebhookPostDtoEventTypesItem",
    "WebhookPutDto",
    "WebhookPutDtoEventTypesItem",
    "WebhookSummaryGetDto",
    "WebhookSummaryGetDtoEventTypesItem",
)



from enum import Enum


class ChargingMeterValueDtoUnit(str, Enum):
    A = "A"
    CELCIUS = "Celcius"
    FAHRENHEIT = "Fahrenheit"
    K = "K"
    KVA = "kVA"
    KVAR = "kvar"
    KVARH = "kvarh"
    KW = "kW"
    KWH = "kWh"
    PERCENT = "Percent"
    V = "V"
    VA = "VA"
    VAR = "var"
    VARH = "varh"
    W = "W"
    WH = "Wh"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class LocationEVSEDtoParkingRestrictionsItem(str, Enum):
    CUSTOMERS = "CUSTOMERS"
    DISABLED = "DISABLED"
    EV_ONLY = "EV_ONLY"
    MOTORCYCLES = "MOTORCYCLES"
    PLUGGED = "PLUGGED"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..models.energy_source_dto_source import EnergySourceDtoSource


T = TypeVar("T", bound="EnergySourceDto")


@_attrs_define
class EnergySourceDto:
    """
    Attributes:
        source (Union[Unset, EnergySourceDtoSource]):  Default: EnergySourceDtoSource.NUCLEAR.
        percentage (Union[Unset, int]):
    """

    source: Union[Unset, EnergySourceDtoSource] = EnergySourceDtoSource.NUCLEAR
    percentage: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        source: Union[Unset, str] = UNSET
        if not isinstance(self.source, Unset):
            source = self.source.value

        percentage = self.percentage

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if source is not UNSET:
            field_dict["source"] = source
        if percentage is not UNSET:
            field_dict["percentage"] = percentage

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _source = d.pop("source", UNSET)
        source: Union[Unset, EnergySourceDtoSource]
        if isinstance(_source, Unset) or _source is None:
            source = UNSET
        else:
            source = EnergySourceDtoSource(_source)

        percentage = d.pop("percentage", UNSET)

        energy_source_dto = cls(
            source=source,
            percentage=percentage,
        )

        energy_source_dto.additional_properties = d
        return energy_source_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
from ..models.tariff_price_dto_approval_status import TariffPriceDtoApprovalStatus
import datetime
from typing import Union

if TYPE_CHECKING:
    from ..models.tariff_restriction import TariffRestriction


T = TypeVar("T", bound="TariffPriceDto")


@_attrs_define
class TariffPriceDto:
    """
    Attributes:
        created_timestamp (Union[Unset, datetime.datetime]):
        valid_from (Union[Unset, datetime.datetime]):
        start_tariff (Union[Unset, float]):
        price_per_kwh (Union[Unset, float]):
        price_per_kwh_incl_vat (Union[Unset, float]):
        is_vat_relevant (Union[Unset, bool]):
        parking_tariff (Union[Unset, float]):
        parking_step_size_in_minutes (Union[Unset, int]):
        parking_grace_period_in_minutes (Union[Unset, int]):
        parking_tariff_restrictions (Union[Unset, List['TariffRestriction']]):
        time_tariff (Union[Unset, float]):
        time_step_size_in_minutes (Union[Unset, int]):
        time_grace_period_in_minutes (Union[Unset, int]):
        approval_status (Union[Unset, TariffPriceDtoApprovalStatus]):  Default: TariffPriceDtoApprovalStatus.PENDING.
    """

    created_timestamp: Union[Unset, datetime.datetime] = UNSET
    valid_from: Union[Unset, datetime.datetime] = UNSET
    start_tariff: Union[Unset, float] = UNSET
    price_per_kwh: Union[Unset, float] = UNSET
    price_per_kwh_incl_vat: Union[Unset, float] = UNSET
    is_vat_relevant: Union[Unset, bool] = UNSET
    parking_tariff: Union[Unset, float] = UNSET
    parking_step_size_in_minutes: Union[Unset, int] = UNSET
    parking_grace_period_in_minutes: Union[Unset, int] = UNSET
    parking_tariff_restrictions: Union[Unset, List["TariffRestriction"]] = UNSET
    time_tariff: Union[Unset, float] = UNSET
    time_step_size_in_minutes: Union[Unset, int] = UNSET
    time_grace_period_in_minutes: Union[Unset, int] = UNSET
    approval_status: Union[Unset, TariffPriceDtoApprovalStatus] = (
        TariffPriceDtoApprovalStatus.PENDING
    )
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        created_timestamp: Union[Unset, str] = UNSET
        if not isinstance(self.created_timestamp, Unset):
            created_timestamp = self.created_timestamp.isoformat()

        valid_from: Union[Unset, str] = UNSET
        if not isinstance(self.valid_from, Unset):
            valid_from = self.valid_from.isoformat()

        start_tariff = self.start_tariff

        price_per_kwh = self.price_per_kwh

        price_per_kwh_incl_vat = self.price_per_kwh_incl_vat

        is_vat_relevant = self.is_vat_relevant

        parking_tariff = self.parking_tariff

        parking_step_size_in_minutes = self.parking_step_size_in_minutes

        parking_grace_period_in_minutes = self.parking_grace_period_in_minutes

        parking_tariff_restrictions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.parking_tariff_restrictions, Unset):
            parking_tariff_restrictions = []
            for (
                parking_tariff_restrictions_item_data
            ) in self.parking_tariff_restrictions:
                parking_tariff_restrictions_item = (
                    parking_tariff_restrictions_item_data.to_dict()
                )
                parking_tariff_restrictions.append(parking_tariff_restrictions_item)

        time_tariff = self.time_tariff

        time_step_size_in_minutes = self.time_step_size_in_minutes

        time_grace_period_in_minutes = self.time_grace_period_in_minutes

        approval_status: Union[Unset, str] = UNSET
        if not isinstance(self.approval_status, Unset):
            approval_status = self.approval_status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if created_timestamp is not UNSET:
            field_dict["createdTimestamp"] = created_timestamp
        if valid_from is not UNSET:
            field_dict["validFrom"] = valid_from
        if start_tariff is not UNSET:
            field_dict["startTariff"] = start_tariff
        if price_per_kwh is not UNSET:
            field_dict["pricePerKwh"] = price_per_kwh
        if price_per_kwh_incl_vat is not UNSET:
            field_dict["pricePerKwhInclVat"] = price_per_kwh_incl_vat
        if is_vat_relevant is not UNSET:
            field_dict["isVatRelevant"] = is_vat_relevant
        if parking_tariff is not UNSET:
            field_dict["parkingTariff"] = parking_tariff
        if parking_step_size_in_minutes is not UNSET:
            field_dict["parkingStepSizeInMinutes"] = parking_step_size_in_minutes
        if parking_grace_period_in_minutes is not UNSET:
            field_dict["parkingGracePeriodInMinutes"] = parking_grace_period_in_minutes
        if parking_tariff_restrictions is not UNSET:
            field_dict["parkingTariffRestrictions"] = parking_tariff_restrictions
        if time_tariff is not UNSET:
            field_dict["timeTariff"] = time_tariff
        if time_step_size_in_minutes is not UNSET:
            field_dict["timeStepSizeInMinutes"] = time_step_size_in_minutes
        if time_grace_period_in_minutes is not UNSET:
            field_dict["timeGracePeriodInMinutes"] = time_grace_period_in_minutes
        if approval_status is not UNSET:
            field_dict["approvalStatus"] = approval_status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.tariff_restriction import TariffRestriction

        d = src_dict.copy()
        _created_timestamp = d.pop("createdTimestamp", UNSET)
        created_timestamp: Union[Unset, datetime.datetime]
        if isinstance(_created_timestamp, Unset) or _created_timestamp is None:
            created_timestamp = UNSET
        else:
            created_timestamp = isoparse(_created_timestamp)

        _valid_from = d.pop("validFrom", UNSET)
        valid_from: Union[Unset, datetime.datetime]
        if isinstance(_valid_from, Unset) or _valid_from is None:
            valid_from = UNSET
        else:
            valid_from = isoparse(_valid_from)

        start_tariff = d.pop("startTariff", UNSET)

        price_per_kwh = d.pop("pricePerKwh", UNSET)

        price_per_kwh_incl_vat = d.pop("pricePerKwhInclVat", UNSET)

        is_vat_relevant = d.pop("isVatRelevant", UNSET)

        parking_tariff = d.pop("parkingTariff", UNSET)

        parking_step_size_in_minutes = d.pop("parkingStepSizeInMinutes", UNSET)

        parking_grace_period_in_minutes = d.pop("parkingGracePeriodInMinutes", UNSET)

        parking_tariff_restrictions = []
        _parking_tariff_restrictions = d.pop("parkingTariffRestrictions", UNSET)
        for parking_tariff_restrictions_item_data in _parking_tariff_restrictions or []:
            parking_tariff_restrictions_item = TariffRestriction.from_dict(
                parking_tariff_restrictions_item_data
            )

            parking_tariff_restrictions.append(parking_tariff_restrictions_item)

        time_tariff = d.pop("timeTariff", UNSET)

        time_step_size_in_minutes = d.pop("timeStepSizeInMinutes", UNSET)

        time_grace_period_in_minutes = d.pop("timeGracePeriodInMinutes", UNSET)

        _approval_status = d.pop("approvalStatus", UNSET)
        approval_status: Union[Unset, TariffPriceDtoApprovalStatus]
        if isinstance(_approval_status, Unset) or _approval_status is None:
            approval_status = UNSET
        else:
            approval_status = TariffPriceDtoApprovalStatus(_approval_status)

        tariff_price_dto = cls(
            created_timestamp=created_timestamp,
            valid_from=valid_from,
            start_tariff=start_tariff,
            price_per_kwh=price_per_kwh,
            price_per_kwh_incl_vat=price_per_kwh_incl_vat,
            is_vat_relevant=is_vat_relevant,
            parking_tariff=parking_tariff,
            parking_step_size_in_minutes=parking_step_size_in_minutes,
            parking_grace_period_in_minutes=parking_grace_period_in_minutes,
            parking_tariff_restrictions=parking_tariff_restrictions,
            time_tariff=time_tariff,
            time_step_size_in_minutes=time_step_size_in_minutes,
            time_grace_period_in_minutes=time_grace_period_in_minutes,
            approval_status=approval_status,
        )

        tariff_price_dto.additional_properties = d
        return tariff_price_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.local_token_group_get_dto import LocalTokenGroupGetDto
from ...models.longship_error import LongshipError
from ...types import Unset


def _get_kwargs(
    *,
    skip: Union[Unset, int] = UNSET,
    take: Union[Unset, int] = UNSET,
    search: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["skip"] = skip

    params["take"] = take

    params["search"] = search

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/v1/localtokengroups",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[List["LocalTokenGroupGetDto"], LongshipError]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = []
        _response_200 = response.json()
        for componentsschemaslocal_token_group_get_dto_array_item_data in _response_200:
            componentsschemaslocal_token_group_get_dto_array_item = (
                LocalTokenGroupGetDto.from_dict(
                    componentsschemaslocal_token_group_get_dto_array_item_data
                )
            )

            response_200.append(componentsschemaslocal_token_group_get_dto_array_item)

        return response_200
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = LongshipError.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = LongshipError.from_dict(response.json())

        return response_500
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = LongshipError.from_dict(response.json())

        return response_403
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[List["LocalTokenGroupGetDto"], LongshipError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    skip: Union[Unset, int] = UNSET,
    take: Union[Unset, int] = UNSET,
    search: Union[Unset, str] = UNSET,
) -> Response[Union[List["LocalTokenGroupGetDto"], LongshipError]]:
    """Get a list of localtokengroups.

     Get a paged list of localtokengroups, taken the filters into account.

    Args:
        skip (Union[Unset, int]):
        take (Union[Unset, int]):
        search (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[List['LocalTokenGroupGetDto'], LongshipError]]
    """

    kwargs = _get_kwargs(
        skip=skip,
        take=take,
        search=search,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    skip: Union[Unset, int] = UNSET,
    take: Union[Unset, int] = UNSET,
    search: Union[Unset, str] = UNSET,
) -> Optional[Union[List["LocalTokenGroupGetDto"], LongshipError]]:
    """Get a list of localtokengroups.

     Get a paged list of localtokengroups, taken the filters into account.

    Args:
        skip (Union[Unset, int]):
        take (Union[Unset, int]):
        search (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[List['LocalTokenGroupGetDto'], LongshipError]
    """

    return sync_detailed(
        client=client,
        skip=skip,
        take=take,
        search=search,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    skip: Union[Unset, int] = UNSET,
    take: Union[Unset, int] = UNSET,
    search: Union[Unset, str] = UNSET,
) -> Response[Union[List["LocalTokenGroupGetDto"], LongshipError]]:
    """Get a list of localtokengroups.

     Get a paged list of localtokengroups, taken the filters into account.

    Args:
        skip (Union[Unset, int]):
        take (Union[Unset, int]):
        search (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[List['LocalTokenGroupGetDto'], LongshipError]]
    """

    kwargs = _get_kwargs(
        skip=skip,
        take=take,
        search=search,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    skip: Union[Unset, int] = UNSET,
    take: Union[Unset, int] = UNSET,
    search: Union[Unset, str] = UNSET,
) -> Optional[Union[List["LocalTokenGroupGetDto"], LongshipError]]:
    """Get a list of localtokengroups.

     Get a paged list of localtokengroups, taken the filters into account.

    Args:
        skip (Union[Unset, int]):
        take (Union[Unset, int]):
        search (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[List['LocalTokenGroupGetDto'], LongshipError]
    """

    return (
        await asyncio_detailed(
            client=client,
            skip=skip,
            take=take,
            search=search,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..models.authorization_assertion_dto_status import AuthorizationAssertionDtoStatus
from ..models.authorization_assertion_dto_auth_scenario_type import (
    AuthorizationAssertionDtoAuthScenarioType,
)


T = TypeVar("T", bound="AuthorizationAssertionDto")


@_attrs_define
class AuthorizationAssertionDto:
    """
    Attributes:
        auth_scenario_type (Union[Unset, AuthorizationAssertionDtoAuthScenarioType]):  Default:
            AuthorizationAssertionDtoAuthScenarioType.NONE.
        auth_result (Union[Unset, str]):
        status (Union[Unset, AuthorizationAssertionDtoStatus]):  Default: AuthorizationAssertionDtoStatus.PENDING.
    """

    auth_scenario_type: Union[Unset, AuthorizationAssertionDtoAuthScenarioType] = (
        AuthorizationAssertionDtoAuthScenarioType.NONE
    )
    auth_result: Union[Unset, str] = UNSET
    status: Union[Unset, AuthorizationAssertionDtoStatus] = (
        AuthorizationAssertionDtoStatus.PENDING
    )
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        auth_scenario_type: Union[Unset, str] = UNSET
        if not isinstance(self.auth_scenario_type, Unset):
            auth_scenario_type = self.auth_scenario_type.value

        auth_result = self.auth_result

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if auth_scenario_type is not UNSET:
            field_dict["authScenarioType"] = auth_scenario_type
        if auth_result is not UNSET:
            field_dict["authResult"] = auth_result
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _auth_scenario_type = d.pop("authScenarioType", UNSET)
        auth_scenario_type: Union[Unset, AuthorizationAssertionDtoAuthScenarioType]
        if isinstance(_auth_scenario_type, Unset) or _auth_scenario_type is None:
            auth_scenario_type = UNSET
        else:
            auth_scenario_type = AuthorizationAssertionDtoAuthScenarioType(
                _auth_scenario_type
            )

        auth_result = d.pop("authResult", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, AuthorizationAssertionDtoStatus]
        if isinstance(_status, Unset) or _status is None:
            status = UNSET
        else:
            status = AuthorizationAssertionDtoStatus(_status)

        authorization_assertion_dto = cls(
            auth_scenario_type=auth_scenario_type,
            auth_result=auth_result,
            status=status,
        )

        authorization_assertion_dto.additional_properties = d
        return authorization_assertion_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.webhook_post_dto_event_types_item import WebhookPostDtoEventTypesItem
from typing import Union

if TYPE_CHECKING:
    from ..models.webhook_header_dto import WebhookHeaderDto


T = TypeVar("T", bound="WebhookPostDto")


@_attrs_define
class WebhookPostDto:
    """
    Attributes:
        name (Union[Unset, str]):
        ou_code (Union[Unset, str]):
        enabled (Union[Unset, bool]):
        event_types (Union[Unset, List[WebhookPostDtoEventTypesItem]]):
        headers (Union[Unset, List['WebhookHeaderDto']]):
        url (Union[Unset, str]):
    """

    name: Union[Unset, str] = UNSET
    ou_code: Union[Unset, str] = UNSET
    enabled: Union[Unset, bool] = UNSET
    event_types: Union[Unset, List[WebhookPostDtoEventTypesItem]] = UNSET
    headers: Union[Unset, List["WebhookHeaderDto"]] = UNSET
    url: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        ou_code = self.ou_code

        enabled = self.enabled

        event_types: Union[Unset, List[str]] = UNSET
        if not isinstance(self.event_types, Unset):
            event_types = []
            for event_types_item_data in self.event_types:
                event_types_item = event_types_item_data.value
                event_types.append(event_types_item)

        headers: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.headers, Unset):
            headers = []
            for headers_item_data in self.headers:
                headers_item = headers_item_data.to_dict()
                headers.append(headers_item)

        url = self.url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if ou_code is not UNSET:
            field_dict["ouCode"] = ou_code
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if event_types is not UNSET:
            field_dict["eventTypes"] = event_types
        if headers is not UNSET:
            field_dict["headers"] = headers
        if url is not UNSET:
            field_dict["url"] = url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.webhook_header_dto import WebhookHeaderDto

        d = src_dict.copy()
        name = d.pop("name", UNSET)

        ou_code = d.pop("ouCode", UNSET)

        enabled = d.pop("enabled", UNSET)

        event_types = []
        _event_types = d.pop("eventTypes", UNSET)
        for event_types_item_data in _event_types or []:
            event_types_item = WebhookPostDtoEventTypesItem(event_types_item_data)

            event_types.append(event_types_item)

        headers = []
        _headers = d.pop("headers", UNSET)
        for headers_item_data in _headers or []:
            headers_item = WebhookHeaderDto.from_dict(headers_item_data)

            headers.append(headers_item)

        url = d.pop("url", UNSET)

        webhook_post_dto = cls(
            name=name,
            ou_code=ou_code,
            enabled=enabled,
            event_types=event_types,
            headers=headers,
            url=url,
        )

        webhook_post_dto.additional_properties = d
        return webhook_post_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
from ..models.message_log_dto_ocpp_message_type import MessageLogDtoOcppMessageType
import datetime
from typing import Union
from ..models.message_log_dto_wamp_message_type import MessageLogDtoWampMessageType
from ..models.message_log_dto_direction import MessageLogDtoDirection


T = TypeVar("T", bound="MessageLogDto")


@_attrs_define
class MessageLogDto:
    """
    Attributes:
        id (Union[Unset, str]):
        charge_point_id (Union[Unset, str]):
        message_id (Union[Unset, str]):
        wamp_message_type (Union[Unset, MessageLogDtoWampMessageType]):  Default: MessageLogDtoWampMessageType.UNKNOWN.
        ocpp_message_type (Union[Unset, MessageLogDtoOcppMessageType]):  Default:
            MessageLogDtoOcppMessageType.AUTHORIZE.
        direction (Union[Unset, MessageLogDtoDirection]):  Default: MessageLogDtoDirection.UNKNOWN.
        tenant_id (Union[Unset, str]):
        payload (Union[Unset, str]):
        timestamp (Union[Unset, datetime.datetime]):
    """

    id: Union[Unset, str] = UNSET
    charge_point_id: Union[Unset, str] = UNSET
    message_id: Union[Unset, str] = UNSET
    wamp_message_type: Union[Unset, MessageLogDtoWampMessageType] = (
        MessageLogDtoWampMessageType.UNKNOWN
    )
    ocpp_message_type: Union[Unset, MessageLogDtoOcppMessageType] = (
        MessageLogDtoOcppMessageType.AUTHORIZE
    )
    direction: Union[Unset, MessageLogDtoDirection] = MessageLogDtoDirection.UNKNOWN
    tenant_id: Union[Unset, str] = UNSET
    payload: Union[Unset, str] = UNSET
    timestamp: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        charge_point_id = self.charge_point_id

        message_id = self.message_id

        wamp_message_type: Union[Unset, str] = UNSET
        if not isinstance(self.wamp_message_type, Unset):
            wamp_message_type = self.wamp_message_type.value

        ocpp_message_type: Union[Unset, str] = UNSET
        if not isinstance(self.ocpp_message_type, Unset):
            ocpp_message_type = self.ocpp_message_type.value

        direction: Union[Unset, str] = UNSET
        if not isinstance(self.direction, Unset):
            direction = self.direction.value

        tenant_id = self.tenant_id

        payload = self.payload

        timestamp: Union[Unset, str] = UNSET
        if not isinstance(self.timestamp, Unset):
            timestamp = self.timestamp.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if charge_point_id is not UNSET:
            field_dict["chargePointId"] = charge_point_id
        if message_id is not UNSET:
            field_dict["messageId"] = message_id
        if wamp_message_type is not UNSET:
            field_dict["wampMessageType"] = wamp_message_type
        if ocpp_message_type is not UNSET:
            field_dict["ocppMessageType"] = ocpp_message_type
        if direction is not UNSET:
            field_dict["direction"] = direction
        if tenant_id is not UNSET:
            field_dict["tenantId"] = tenant_id
        if payload is not UNSET:
            field_dict["payload"] = payload
        if timestamp is not UNSET:
            field_dict["timestamp"] = timestamp

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        charge_point_id = d.pop("chargePointId", UNSET)

        message_id = d.pop("messageId", UNSET)

        _wamp_message_type = d.pop("wampMessageType", UNSET)
        wamp_message_type: Union[Unset, MessageLogDtoWampMessageType]
        if isinstance(_wamp_message_type, Unset) or _wamp_message_type is None:
            wamp_message_type = UNSET
        else:
            wamp_message_type = MessageLogDtoWampMessageType(_wamp_message_type)

        _ocpp_message_type = d.pop("ocppMessageType", UNSET)
        ocpp_message_type: Union[Unset, MessageLogDtoOcppMessageType]
        if isinstance(_ocpp_message_type, Unset) or _ocpp_message_type is None:
            ocpp_message_type = UNSET
        else:
            ocpp_message_type = MessageLogDtoOcppMessageType(_ocpp_message_type)

        _direction = d.pop("direction", UNSET)
        direction: Union[Unset, MessageLogDtoDirection]
        if isinstance(_direction, Unset) or _direction is None:
            direction = UNSET
        else:
            direction = MessageLogDtoDirection(_direction)

        tenant_id = d.pop("tenantId", UNSET)

        payload = d.pop("payload", UNSET)

        _timestamp = d.pop("timestamp", UNSET)
        timestamp: Union[Unset, datetime.datetime]
        if isinstance(_timestamp, Unset) or _timestamp is None:
            timestamp = UNSET
        else:
            timestamp = isoparse(_timestamp)

        message_log_dto = cls(
            id=id,
            charge_point_id=charge_point_id,
            message_id=message_id,
            wamp_message_type=wamp_message_type,
            ocpp_message_type=ocpp_message_type,
            direction=direction,
            tenant_id=tenant_id,
            payload=payload,
            timestamp=timestamp,
        )

        message_log_dto.additional_properties = d
        return message_log_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.cdr_dto_approval_status import CdrDtoApprovalStatus
from dateutil.parser import isoparse
from ..models.cdr_dto_financial_type import CdrDtoFinancialType
import datetime
from typing import Union

if TYPE_CHECKING:
    from ..models.cdr_location_dto import CdrLocationDto
    from ..models.cdr_started_by_info_dto import CdrStartedByInfoDto
    from ..models.charging_period_dto import ChargingPeriodDto
    from ..models.price_info_dto import PriceInfoDto


T = TypeVar("T", bound="CdrDto")


@_attrs_define
class CdrDto:
    """
    Attributes:
        id (Union[Unset, str]):
        tenant_id (Union[Unset, str]):
        charge_point_id (Union[Unset, str]):
        connector_id (Union[Unset, int]):
        cdr_location (Union[Unset, CdrLocationDto]):
        start_datetime (Union[Unset, datetime.datetime]):
        end_date_time (Union[Unset, datetime.datetime]):
        session_id (Union[Unset, str]):
        token (Union[Unset, str]):
        started_by_info (Union[Unset, CdrStartedByInfoDto]):
        total_energy_in_kwh (Union[Unset, float]):
        total_time_in_hours (Union[Unset, float]):
        charging_periods (Union[Unset, List['ChargingPeriodDto']]):
        total_price (Union[Unset, float]):
        created (Union[Unset, datetime.datetime]):
        last_updated (Union[Unset, datetime.datetime]):
        ou (Union[Unset, str]):
        ou_id (Union[Unset, str]):
        ou_name (Union[Unset, str]):
        tariff_id (Union[Unset, str]):
        tariff_name (Union[Unset, str]):
        start_tariff (Union[Unset, float]):
        tariff_price (Union[Unset, float]):
        charging_time_tariff (Union[Unset, float]):
        parking_time_tariff (Union[Unset, float]):
        price_info (Union[Unset, PriceInfoDto]):
        local_start_date_time (Union[Unset, datetime.datetime]):
        local_end_date_time (Union[Unset, datetime.datetime]):
        approval_status (Union[Unset, CdrDtoApprovalStatus]):  Default: CdrDtoApprovalStatus.APPROVED.
        financial_type (Union[Unset, CdrDtoFinancialType]):  Default: CdrDtoFinancialType.DEBIT.
        debit_cdr_id (Union[Unset, str]):
    """

    id: Union[Unset, str] = UNSET
    tenant_id: Union[Unset, str] = UNSET
    charge_point_id: Union[Unset, str] = UNSET
    connector_id: Union[Unset, int] = UNSET
    cdr_location: Union[Unset, "CdrLocationDto"] = UNSET
    start_datetime: Union[Unset, datetime.datetime] = UNSET
    end_date_time: Union[Unset, datetime.datetime] = UNSET
    session_id: Union[Unset, str] = UNSET
    token: Union[Unset, str] = UNSET
    started_by_info: Union[Unset, "CdrStartedByInfoDto"] = UNSET
    total_energy_in_kwh: Union[Unset, float] = UNSET
    total_time_in_hours: Union[Unset, float] = UNSET
    charging_periods: Union[Unset, List["ChargingPeriodDto"]] = UNSET
    total_price: Union[Unset, float] = UNSET
    created: Union[Unset, datetime.datetime] = UNSET
    last_updated: Union[Unset, datetime.datetime] = UNSET
    ou: Union[Unset, str] = UNSET
    ou_id: Union[Unset, str] = UNSET
    ou_name: Union[Unset, str] = UNSET
    tariff_id: Union[Unset, str] = UNSET
    tariff_name: Union[Unset, str] = UNSET
    start_tariff: Union[Unset, float] = UNSET
    tariff_price: Union[Unset, float] = UNSET
    charging_time_tariff: Union[Unset, float] = UNSET
    parking_time_tariff: Union[Unset, float] = UNSET
    price_info: Union[Unset, "PriceInfoDto"] = UNSET
    local_start_date_time: Union[Unset, datetime.datetime] = UNSET
    local_end_date_time: Union[Unset, datetime.datetime] = UNSET
    approval_status: Union[Unset, CdrDtoApprovalStatus] = CdrDtoApprovalStatus.APPROVED
    financial_type: Union[Unset, CdrDtoFinancialType] = CdrDtoFinancialType.DEBIT
    debit_cdr_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        tenant_id = self.tenant_id

        charge_point_id = self.charge_point_id

        connector_id = self.connector_id

        cdr_location: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.cdr_location, Unset):
            cdr_location = self.cdr_location.to_dict()

        start_datetime: Union[Unset, str] = UNSET
        if not isinstance(self.start_datetime, Unset):
            start_datetime = self.start_datetime.isoformat()

        end_date_time: Union[Unset, str] = UNSET
        if not isinstance(self.end_date_time, Unset):
            end_date_time = self.end_date_time.isoformat()

        session_id = self.session_id

        token = self.token

        started_by_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.started_by_info, Unset):
            started_by_info = self.started_by_info.to_dict()

        total_energy_in_kwh = self.total_energy_in_kwh

        total_time_in_hours = self.total_time_in_hours

        charging_periods: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.charging_periods, Unset):
            charging_periods = []
            for charging_periods_item_data in self.charging_periods:
                charging_periods_item = charging_periods_item_data.to_dict()
                charging_periods.append(charging_periods_item)

        total_price = self.total_price

        created: Union[Unset, str] = UNSET
        if not isinstance(self.created, Unset):
            created = self.created.isoformat()

        last_updated: Union[Unset, str] = UNSET
        if not isinstance(self.last_updated, Unset):
            last_updated = self.last_updated.isoformat()

        ou = self.ou

        ou_id = self.ou_id

        ou_name = self.ou_name

        tariff_id = self.tariff_id

        tariff_name = self.tariff_name

        start_tariff = self.start_tariff

        tariff_price = self.tariff_price

        charging_time_tariff = self.charging_time_tariff

        parking_time_tariff = self.parking_time_tariff

        price_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.price_info, Unset):
            price_info = self.price_info.to_dict()

        local_start_date_time: Union[Unset, str] = UNSET
        if not isinstance(self.local_start_date_time, Unset):
            local_start_date_time = self.local_start_date_time.isoformat()

        local_end_date_time: Union[Unset, str] = UNSET
        if not isinstance(self.local_end_date_time, Unset):
            local_end_date_time = self.local_end_date_time.isoformat()

        approval_status: Union[Unset, str] = UNSET
        if not isinstance(self.approval_status, Unset):
            approval_status = self.approval_status.value

        financial_type: Union[Unset, str] = UNSET
        if not isinstance(self.financial_type, Unset):
            financial_type = self.financial_type.value

        debit_cdr_id = self.debit_cdr_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if tenant_id is not UNSET:
            field_dict["tenantId"] = tenant_id
        if charge_point_id is not UNSET:
            field_dict["chargePointId"] = charge_point_id
        if connector_id is not UNSET:
            field_dict["connectorId"] = connector_id
        if cdr_location is not UNSET:
            field_dict["cdrLocation"] = cdr_location
        if start_datetime is not UNSET:
            field_dict["startDatetime"] = start_datetime
        if end_date_time is not UNSET:
            field_dict["endDateTime"] = end_date_time
        if session_id is not UNSET:
            field_dict["sessionId"] = session_id
        if token is not UNSET:
            field_dict["token"] = token
        if started_by_info is not UNSET:
            field_dict["startedByInfo"] = started_by_info
        if total_energy_in_kwh is not UNSET:
            field_dict["totalEnergyInKwh"] = total_energy_in_kwh
        if total_time_in_hours is not UNSET:
            field_dict["totalTimeInHours"] = total_time_in_hours
        if charging_periods is not UNSET:
            field_dict["chargingPeriods"] = charging_periods
        if total_price is not UNSET:
            field_dict["totalPrice"] = total_price
        if created is not UNSET:
            field_dict["created"] = created
        if last_updated is not UNSET:
            field_dict["lastUpdated"] = last_updated
        if ou is not UNSET:
            field_dict["ou"] = ou
        if ou_id is not UNSET:
            field_dict["ouId"] = ou_id
        if ou_name is not UNSET:
            field_dict["ouName"] = ou_name
        if tariff_id is not UNSET:
            field_dict["tariffId"] = tariff_id
        if tariff_name is not UNSET:
            field_dict["tariffName"] = tariff_name
        if start_tariff is not UNSET:
            field_dict["startTariff"] = start_tariff
        if tariff_price is not UNSET:
            field_dict["tariffPrice"] = tariff_price
        if charging_time_tariff is not UNSET:
            field_dict["chargingTimeTariff"] = charging_time_tariff
        if parking_time_tariff is not UNSET:
            field_dict["parkingTimeTariff"] = parking_time_tariff
        if price_info is not UNSET:
            field_dict["priceInfo"] = price_info
        if local_start_date_time is not UNSET:
            field_dict["localStartDateTime"] = local_start_date_time
        if local_end_date_time is not UNSET:
            field_dict["localEndDateTime"] = local_end_date_time
        if approval_status is not UNSET:
            field_dict["approvalStatus"] = approval_status
        if financial_type is not UNSET:
            field_dict["financialType"] = financial_type
        if debit_cdr_id is not UNSET:
            field_dict["debitCdrId"] = debit_cdr_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.cdr_location_dto import CdrLocationDto
        from ..models.cdr_started_by_info_dto import CdrStartedByInfoDto
        from ..models.charging_period_dto import ChargingPeriodDto
        from ..models.price_info_dto import PriceInfoDto

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        tenant_id = d.pop("tenantId", UNSET)

        charge_point_id = d.pop("chargePointId", UNSET)

        connector_id = d.pop("connectorId", UNSET)

        _cdr_location = d.pop("cdrLocation", UNSET)
        cdr_location: Union[Unset, CdrLocationDto]
        if isinstance(_cdr_location, Unset) or _cdr_location is None:
            cdr_location = UNSET
        else:
            cdr_location = CdrLocationDto.from_dict(_cdr_location)

        _start_datetime = d.pop("startDatetime", UNSET)
        start_datetime: Union[Unset, datetime.datetime]
        if isinstance(_start_datetime, Unset) or _start_datetime is None:
            start_datetime = UNSET
        else:
            start_datetime = isoparse(_start_datetime)

        _end_date_time = d.pop("endDateTime", UNSET)
        end_date_time: Union[Unset, datetime.datetime]
        if isinstance(_end_date_time, Unset) or _end_date_time is None:
            end_date_time = UNSET
        else:
            end_date_time = isoparse(_end_date_time)

        session_id = d.pop("sessionId", UNSET)

        token = d.pop("token", UNSET)

        _started_by_info = d.pop("startedByInfo", UNSET)
        started_by_info: Union[Unset, CdrStartedByInfoDto]
        if isinstance(_started_by_info, Unset) or _started_by_info is None:
            started_by_info = UNSET
        else:
            started_by_info = CdrStartedByInfoDto.from_dict(_started_by_info)

        total_energy_in_kwh = d.pop("totalEnergyInKwh", UNSET)

        total_time_in_hours = d.pop("totalTimeInHours", UNSET)

        charging_periods = []
        _charging_periods = d.pop("chargingPeriods", UNSET)
        for charging_periods_item_data in _charging_periods or []:
            charging_periods_item = ChargingPeriodDto.from_dict(
                charging_periods_item_data
            )

            charging_periods.append(charging_periods_item)

        total_price = d.pop("totalPrice", UNSET)

        _created = d.pop("created", UNSET)
        created: Union[Unset, datetime.datetime]
        if isinstance(_created, Unset) or _created is None:
            created = UNSET
        else:
            created = isoparse(_created)

        _last_updated = d.pop("lastUpdated", UNSET)
        last_updated: Union[Unset, datetime.datetime]
        if isinstance(_last_updated, Unset) or _last_updated is None:
            last_updated = UNSET
        else:
            last_updated = isoparse(_last_updated)

        ou = d.pop("ou", UNSET)

        ou_id = d.pop("ouId", UNSET)

        ou_name = d.pop("ouName", UNSET)

        tariff_id = d.pop("tariffId", UNSET)

        tariff_name = d.pop("tariffName", UNSET)

        start_tariff = d.pop("startTariff", UNSET)

        tariff_price = d.pop("tariffPrice", UNSET)

        charging_time_tariff = d.pop("chargingTimeTariff", UNSET)

        parking_time_tariff = d.pop("parkingTimeTariff", UNSET)

        _price_info = d.pop("priceInfo", UNSET)
        price_info: Union[Unset, PriceInfoDto]
        if isinstance(_price_info, Unset) or _price_info is None:
            price_info = UNSET
        else:
            price_info = PriceInfoDto.from_dict(_price_info)

        _local_start_date_time = d.pop("localStartDateTime", UNSET)
        local_start_date_time: Union[Unset, datetime.datetime]
        if isinstance(_local_start_date_time, Unset) or _local_start_date_time is None:
            local_start_date_time = UNSET
        else:
            local_start_date_time = isoparse(_local_start_date_time)

        _local_end_date_time = d.pop("localEndDateTime", UNSET)
        local_end_date_time: Union[Unset, datetime.datetime]
        if isinstance(_local_end_date_time, Unset) or _local_end_date_time is None:
            local_end_date_time = UNSET
        else:
            local_end_date_time = isoparse(_local_end_date_time)

        _approval_status = d.pop("approvalStatus", UNSET)
        approval_status: Union[Unset, CdrDtoApprovalStatus]
        if isinstance(_approval_status, Unset) or _approval_status is None:
            approval_status = UNSET
        else:
            approval_status = CdrDtoApprovalStatus(_approval_status)

        _financial_type = d.pop("financialType", UNSET)
        financial_type: Union[Unset, CdrDtoFinancialType]
        if isinstance(_financial_type, Unset) or _financial_type is None:
            financial_type = UNSET
        else:
            financial_type = CdrDtoFinancialType(_financial_type)

        debit_cdr_id = d.pop("debitCdrId", UNSET)

        cdr_dto = cls(
            id=id,
            tenant_id=tenant_id,
            charge_point_id=charge_point_id,
            connector_id=connector_id,
            cdr_location=cdr_location,
            start_datetime=start_datetime,
            end_date_time=end_date_time,
            session_id=session_id,
            token=token,
            started_by_info=started_by_info,
            total_energy_in_kwh=total_energy_in_kwh,
            total_time_in_hours=total_time_in_hours,
            charging_periods=charging_periods,
            total_price=total_price,
            created=created,
            last_updated=last_updated,
            ou=ou,
            ou_id=ou_id,
            ou_name=ou_name,
            tariff_id=tariff_id,
            tariff_name=tariff_name,
            start_tariff=start_tariff,
            tariff_price=tariff_price,
            charging_time_tariff=charging_time_tariff,
            parking_time_tariff=parking_time_tariff,
            price_info=price_info,
            local_start_date_time=local_start_date_time,
            local_end_date_time=local_end_date_time,
            approval_status=approval_status,
            financial_type=financial_type,
            debit_cdr_id=debit_cdr_id,
        )

        cdr_dto.additional_properties = d
        return cdr_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class StatusScheduleDtoStatus(str, Enum):
    AVAILABLE = "AVAILABLE"
    BLOCKED = "BLOCKED"
    CHARGING = "CHARGING"
    INOPERATIVE = "INOPERATIVE"
    OUTOFORDER = "OUTOFORDER"
    PLANNED = "PLANNED"
    REMOVED = "REMOVED"
    RESERVED = "RESERVED"
    UNKNOWN = "UNKNOWN"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.get_composite_schedule_request_charging_rate_unit import (
    GetCompositeScheduleRequestChargingRateUnit,
)
from typing import Union


T = TypeVar("T", bound="GetCompositeScheduleRequest")


@_attrs_define
class GetCompositeScheduleRequest:
    """
    Attributes:
        connector_id (int):
        duration (int):
        charging_rate_unit (Union[Unset, GetCompositeScheduleRequestChargingRateUnit]):  Default:
            GetCompositeScheduleRequestChargingRateUnit.A.
    """

    connector_id: int
    duration: int
    charging_rate_unit: Union[Unset, GetCompositeScheduleRequestChargingRateUnit] = (
        GetCompositeScheduleRequestChargingRateUnit.A
    )
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        connector_id = self.connector_id

        duration = self.duration

        charging_rate_unit: Union[Unset, str] = UNSET
        if not isinstance(self.charging_rate_unit, Unset):
            charging_rate_unit = self.charging_rate_unit.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "connectorId": connector_id,
                "duration": duration,
            }
        )
        if charging_rate_unit is not UNSET:
            field_dict["chargingRateUnit"] = charging_rate_unit

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        connector_id = d.pop("connectorId")

        duration = d.pop("duration")

        _charging_rate_unit = d.pop("chargingRateUnit", UNSET)
        charging_rate_unit: Union[Unset, GetCompositeScheduleRequestChargingRateUnit]
        if isinstance(_charging_rate_unit, Unset) or _charging_rate_unit is None:
            charging_rate_unit = UNSET
        else:
            charging_rate_unit = GetCompositeScheduleRequestChargingRateUnit(
                _charging_rate_unit
            )

        get_composite_schedule_request = cls(
            connector_id=connector_id,
            duration=duration,
            charging_rate_unit=charging_rate_unit,
        )

        get_composite_schedule_request.additional_properties = d
        return get_composite_schedule_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.chargepoint_dto_connectivity_status import (
    ChargepointDtoConnectivityStatus,
)
from dateutil.parser import isoparse
from typing import cast
import datetime
from typing import Union

if TYPE_CHECKING:
    from ..models.chargepoint_evse_dto import ChargepointEVSEDto


T = TypeVar("T", bound="ChargepointDto")


@_attrs_define
class ChargepointDto:
    """
    Attributes:
        id (Union[Unset, str]):
        charge_point_id (Union[Unset, str]):
        date_deleted (Union[Unset, datetime.datetime]):
        display_name (Union[Unset, str]):
        roaming_name (Union[Unset, str]):
        charge_box_serial_number (Union[Unset, str]):
        charge_point_model (Union[Unset, str]):
        charge_point_serial_number (Union[Unset, str]):
        charge_point_vendor (Union[Unset, str]):
        firmware_version (Union[Unset, str]):
        connectivity_status (Union[Unset, ChargepointDtoConnectivityStatus]):  Default:
            ChargepointDtoConnectivityStatus.ONLINE.
        iccid (Union[Unset, str]):
        imsi (Union[Unset, str]):
        meter_serial_number (Union[Unset, str]):
        meter_type (Union[Unset, str]):
        tenant_id (Union[Unset, str]):
        evses (Union[Unset, List['ChargepointEVSEDto']]):
        is_roaming (Union[Unset, bool]):
        has_guest_usage (Union[Unset, bool]):
        location_id (Union[Unset, str]):
        allow_any_token (Union[Unset, bool]):
        date_created (Union[Unset, datetime.datetime]):
        updated (Union[Unset, datetime.datetime]):
        ou (Union[Unset, str]):
        ou_id (Union[Unset, str]):
        ou_name (Union[Unset, str]):
        tariff_id (Union[Unset, str]):
        tariff_name (Union[Unset, str]):
        start_tariff (Union[Unset, float]):
        tariff_price (Union[Unset, float]):
        sim_card_number (Union[Unset, str]):
        token_groups (Union[Unset, List[str]]):
        is_new (Union[Unset, bool]):
        has_reimbursement (Union[Unset, bool]):
        reimburse_tariff_id (Union[Unset, str]):
        reimburse_tariff_name (Union[Unset, str]):
        reimburse_tariff_price (Union[Unset, float]):
        reimburse_uid (Union[Unset, str]):
        reimburse_token_id (Union[Unset, str]):
        reimburse_ou (Union[Unset, str]):
        use_tenant_fee (Union[Unset, bool]):
        max_capacity_in_kw (Union[Unset, float]):
    """

    id: Union[Unset, str] = UNSET
    charge_point_id: Union[Unset, str] = UNSET
    date_deleted: Union[Unset, datetime.datetime] = UNSET
    display_name: Union[Unset, str] = UNSET
    roaming_name: Union[Unset, str] = UNSET
    charge_box_serial_number: Union[Unset, str] = UNSET
    charge_point_model: Union[Unset, str] = UNSET
    charge_point_serial_number: Union[Unset, str] = UNSET
    charge_point_vendor: Union[Unset, str] = UNSET
    firmware_version: Union[Unset, str] = UNSET
    connectivity_status: Union[Unset, ChargepointDtoConnectivityStatus] = (
        ChargepointDtoConnectivityStatus.ONLINE
    )
    iccid: Union[Unset, str] = UNSET
    imsi: Union[Unset, str] = UNSET
    meter_serial_number: Union[Unset, str] = UNSET
    meter_type: Union[Unset, str] = UNSET
    tenant_id: Union[Unset, str] = UNSET
    evses: Union[Unset, List["ChargepointEVSEDto"]] = UNSET
    is_roaming: Union[Unset, bool] = UNSET
    has_guest_usage: Union[Unset, bool] = UNSET
    location_id: Union[Unset, str] = UNSET
    allow_any_token: Union[Unset, bool] = UNSET
    date_created: Union[Unset, datetime.datetime] = UNSET
    updated: Union[Unset, datetime.datetime] = UNSET
    ou: Union[Unset, str] = UNSET
    ou_id: Union[Unset, str] = UNSET
    ou_name: Union[Unset, str] = UNSET
    tariff_id: Union[Unset, str] = UNSET
    tariff_name: Union[Unset, str] = UNSET
    start_tariff: Union[Unset, float] = UNSET
    tariff_price: Union[Unset, float] = UNSET
    sim_card_number: Union[Unset, str] = UNSET
    token_groups: Union[Unset, List[str]] = UNSET
    is_new: Union[Unset, bool] = UNSET
    has_reimbursement: Union[Unset, bool] = UNSET
    reimburse_tariff_id: Union[Unset, str] = UNSET
    reimburse_tariff_name: Union[Unset, str] = UNSET
    reimburse_tariff_price: Union[Unset, float] = UNSET
    reimburse_uid: Union[Unset, str] = UNSET
    reimburse_token_id: Union[Unset, str] = UNSET
    reimburse_ou: Union[Unset, str] = UNSET
    use_tenant_fee: Union[Unset, bool] = UNSET
    max_capacity_in_kw: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        charge_point_id = self.charge_point_id

        date_deleted: Union[Unset, str] = UNSET
        if not isinstance(self.date_deleted, Unset):
            date_deleted = self.date_deleted.isoformat()

        display_name = self.display_name

        roaming_name = self.roaming_name

        charge_box_serial_number = self.charge_box_serial_number

        charge_point_model = self.charge_point_model

        charge_point_serial_number = self.charge_point_serial_number

        charge_point_vendor = self.charge_point_vendor

        firmware_version = self.firmware_version

        connectivity_status: Union[Unset, str] = UNSET
        if not isinstance(self.connectivity_status, Unset):
            connectivity_status = self.connectivity_status.value

        iccid = self.iccid

        imsi = self.imsi

        meter_serial_number = self.meter_serial_number

        meter_type = self.meter_type

        tenant_id = self.tenant_id

        evses: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.evses, Unset):
            evses = []
            for evses_item_data in self.evses:
                evses_item = evses_item_data.to_dict()
                evses.append(evses_item)

        is_roaming = self.is_roaming

        has_guest_usage = self.has_guest_usage

        location_id = self.location_id

        allow_any_token = self.allow_any_token

        date_created: Union[Unset, str] = UNSET
        if not isinstance(self.date_created, Unset):
            date_created = self.date_created.isoformat()

        updated: Union[Unset, str] = UNSET
        if not isinstance(self.updated, Unset):
            updated = self.updated.isoformat()

        ou = self.ou

        ou_id = self.ou_id

        ou_name = self.ou_name

        tariff_id = self.tariff_id

        tariff_name = self.tariff_name

        start_tariff = self.start_tariff

        tariff_price = self.tariff_price

        sim_card_number = self.sim_card_number

        token_groups: Union[Unset, List[str]] = UNSET
        if not isinstance(self.token_groups, Unset):
            token_groups = self.token_groups

        is_new = self.is_new

        has_reimbursement = self.has_reimbursement

        reimburse_tariff_id = self.reimburse_tariff_id

        reimburse_tariff_name = self.reimburse_tariff_name

        reimburse_tariff_price = self.reimburse_tariff_price

        reimburse_uid = self.reimburse_uid

        reimburse_token_id = self.reimburse_token_id

        reimburse_ou = self.reimburse_ou

        use_tenant_fee = self.use_tenant_fee

        max_capacity_in_kw = self.max_capacity_in_kw

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if charge_point_id is not UNSET:
            field_dict["chargePointId"] = charge_point_id
        if date_deleted is not UNSET:
            field_dict["dateDeleted"] = date_deleted
        if display_name is not UNSET:
            field_dict["displayName"] = display_name
        if roaming_name is not UNSET:
            field_dict["roamingName"] = roaming_name
        if charge_box_serial_number is not UNSET:
            field_dict["chargeBoxSerialNumber"] = charge_box_serial_number
        if charge_point_model is not UNSET:
            field_dict["chargePointModel"] = charge_point_model
        if charge_point_serial_number is not UNSET:
            field_dict["chargePointSerialNumber"] = charge_point_serial_number
        if charge_point_vendor is not UNSET:
            field_dict["chargePointVendor"] = charge_point_vendor
        if firmware_version is not UNSET:
            field_dict["firmwareVersion"] = firmware_version
        if connectivity_status is not UNSET:
            field_dict["connectivityStatus"] = connectivity_status
        if iccid is not UNSET:
            field_dict["iccid"] = iccid
        if imsi is not UNSET:
            field_dict["imsi"] = imsi
        if meter_serial_number is not UNSET:
            field_dict["meterSerialNumber"] = meter_serial_number
        if meter_type is not UNSET:
            field_dict["meterType"] = meter_type
        if tenant_id is not UNSET:
            field_dict["tenantId"] = tenant_id
        if evses is not UNSET:
            field_dict["evses"] = evses
        if is_roaming is not UNSET:
            field_dict["isRoaming"] = is_roaming
        if has_guest_usage is not UNSET:
            field_dict["hasGuestUsage"] = has_guest_usage
        if location_id is not UNSET:
            field_dict["locationId"] = location_id
        if allow_any_token is not UNSET:
            field_dict["allowAnyToken"] = allow_any_token
        if date_created is not UNSET:
            field_dict["dateCreated"] = date_created
        if updated is not UNSET:
            field_dict["updated"] = updated
        if ou is not UNSET:
            field_dict["ou"] = ou
        if ou_id is not UNSET:
            field_dict["ouId"] = ou_id
        if ou_name is not UNSET:
            field_dict["ouName"] = ou_name
        if tariff_id is not UNSET:
            field_dict["tariffId"] = tariff_id
        if tariff_name is not UNSET:
            field_dict["tariffName"] = tariff_name
        if start_tariff is not UNSET:
            field_dict["startTariff"] = start_tariff
        if tariff_price is not UNSET:
            field_dict["tariffPrice"] = tariff_price
        if sim_card_number is not UNSET:
            field_dict["simCardNumber"] = sim_card_number
        if token_groups is not UNSET:
            field_dict["tokenGroups"] = token_groups
        if is_new is not UNSET:
            field_dict["isNew"] = is_new
        if has_reimbursement is not UNSET:
            field_dict["hasReimbursement"] = has_reimbursement
        if reimburse_tariff_id is not UNSET:
            field_dict["reimburseTariffId"] = reimburse_tariff_id
        if reimburse_tariff_name is not UNSET:
            field_dict["reimburseTariffName"] = reimburse_tariff_name
        if reimburse_tariff_price is not UNSET:
            field_dict["reimburseTariffPrice"] = reimburse_tariff_price
        if reimburse_uid is not UNSET:
            field_dict["reimburseUID"] = reimburse_uid
        if reimburse_token_id is not UNSET:
            field_dict["reimburseTokenId"] = reimburse_token_id
        if reimburse_ou is not UNSET:
            field_dict["reimburseOU"] = reimburse_ou
        if use_tenant_fee is not UNSET:
            field_dict["useTenantFee"] = use_tenant_fee
        if max_capacity_in_kw is not UNSET:
            field_dict["maxCapacityInKw"] = max_capacity_in_kw

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.chargepoint_evse_dto import ChargepointEVSEDto

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        charge_point_id = d.pop("chargePointId", UNSET)

        _date_deleted = d.pop("dateDeleted", UNSET)
        date_deleted: Union[Unset, datetime.datetime]
        if isinstance(_date_deleted, Unset) or _date_deleted is None:
            date_deleted = UNSET
        else:
            date_deleted = isoparse(_date_deleted)

        display_name = d.pop("displayName", UNSET)

        roaming_name = d.pop("roamingName", UNSET)

        charge_box_serial_number = d.pop("chargeBoxSerialNumber", UNSET)

        charge_point_model = d.pop("chargePointModel", UNSET)

        charge_point_serial_number = d.pop("chargePointSerialNumber", UNSET)

        charge_point_vendor = d.pop("chargePointVendor", UNSET)

        firmware_version = d.pop("firmwareVersion", UNSET)

        _connectivity_status = d.pop("connectivityStatus", UNSET)
        connectivity_status: Union[Unset, ChargepointDtoConnectivityStatus]
        if isinstance(_connectivity_status, Unset) or _connectivity_status is None:
            connectivity_status = UNSET
        else:
            connectivity_status = ChargepointDtoConnectivityStatus(_connectivity_status)

        iccid = d.pop("iccid", UNSET)

        imsi = d.pop("imsi", UNSET)

        meter_serial_number = d.pop("meterSerialNumber", UNSET)

        meter_type = d.pop("meterType", UNSET)

        tenant_id = d.pop("tenantId", UNSET)

        evses = []
        _evses = d.pop("evses", UNSET)
        for evses_item_data in _evses or []:
            evses_item = ChargepointEVSEDto.from_dict(evses_item_data)

            evses.append(evses_item)

        is_roaming = d.pop("isRoaming", UNSET)

        has_guest_usage = d.pop("hasGuestUsage", UNSET)

        location_id = d.pop("locationId", UNSET)

        allow_any_token = d.pop("allowAnyToken", UNSET)

        _date_created = d.pop("dateCreated", UNSET)
        date_created: Union[Unset, datetime.datetime]
        if isinstance(_date_created, Unset) or _date_created is None:
            date_created = UNSET
        else:
            date_created = isoparse(_date_created)

        _updated = d.pop("updated", UNSET)
        updated: Union[Unset, datetime.datetime]
        if isinstance(_updated, Unset) or _updated is None:
            updated = UNSET
        else:
            updated = isoparse(_updated)

        ou = d.pop("ou", UNSET)

        ou_id = d.pop("ouId", UNSET)

        ou_name = d.pop("ouName", UNSET)

        tariff_id = d.pop("tariffId", UNSET)

        tariff_name = d.pop("tariffName", UNSET)

        start_tariff = d.pop("startTariff", UNSET)

        tariff_price = d.pop("tariffPrice", UNSET)

        sim_card_number = d.pop("simCardNumber", UNSET)

        token_groups = cast(List[str], d.pop("tokenGroups", UNSET))

        is_new = d.pop("isNew", UNSET)

        has_reimbursement = d.pop("hasReimbursement", UNSET)

        reimburse_tariff_id = d.pop("reimburseTariffId", UNSET)

        reimburse_tariff_name = d.pop("reimburseTariffName", UNSET)

        reimburse_tariff_price = d.pop("reimburseTariffPrice", UNSET)

        reimburse_uid = d.pop("reimburseUID", UNSET)

        reimburse_token_id = d.pop("reimburseTokenId", UNSET)

        reimburse_ou = d.pop("reimburseOU", UNSET)

        use_tenant_fee = d.pop("useTenantFee", UNSET)

        max_capacity_in_kw = d.pop("maxCapacityInKw", UNSET)

        chargepoint_dto = cls(
            id=id,
            charge_point_id=charge_point_id,
            date_deleted=date_deleted,
            display_name=display_name,
            roaming_name=roaming_name,
            charge_box_serial_number=charge_box_serial_number,
            charge_point_model=charge_point_model,
            charge_point_serial_number=charge_point_serial_number,
            charge_point_vendor=charge_point_vendor,
            firmware_version=firmware_version,
            connectivity_status=connectivity_status,
            iccid=iccid,
            imsi=imsi,
            meter_serial_number=meter_serial_number,
            meter_type=meter_type,
            tenant_id=tenant_id,
            evses=evses,
            is_roaming=is_roaming,
            has_guest_usage=has_guest_usage,
            location_id=location_id,
            allow_any_token=allow_any_token,
            date_created=date_created,
            updated=updated,
            ou=ou,
            ou_id=ou_id,
            ou_name=ou_name,
            tariff_id=tariff_id,
            tariff_name=tariff_name,
            start_tariff=start_tariff,
            tariff_price=tariff_price,
            sim_card_number=sim_card_number,
            token_groups=token_groups,
            is_new=is_new,
            has_reimbursement=has_reimbursement,
            reimburse_tariff_id=reimburse_tariff_id,
            reimburse_tariff_name=reimburse_tariff_name,
            reimburse_tariff_price=reimburse_tariff_price,
            reimburse_uid=reimburse_uid,
            reimburse_token_id=reimburse_token_id,
            reimburse_ou=reimburse_ou,
            use_tenant_fee=use_tenant_fee,
            max_capacity_in_kw=max_capacity_in_kw,
        )

        chargepoint_dto.additional_properties = d
        return chargepoint_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class TariffDtoUsageType(str, Enum):
    BUY = "BUY"
    DEFAULT = "DEFAULT"
    REIMBURSE = "REIMBURSE"
    SELL = "SELL"
    SELLPRIVATE = "SELLPRIVATE"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
import datetime
from typing import Union


T = TypeVar("T", bound="ChargingPeriodDto")


@_attrs_define
class ChargingPeriodDto:
    """
    Attributes:
        timestamp (Union[Unset, datetime.datetime]):
        delta_kwh (Union[Unset, float]):
        absolute_kwh (Union[Unset, float]):
        price (Union[Unset, float]):
        parking_time_minutes (Union[Unset, float]):
    """

    timestamp: Union[Unset, datetime.datetime] = UNSET
    delta_kwh: Union[Unset, float] = UNSET
    absolute_kwh: Union[Unset, float] = UNSET
    price: Union[Unset, float] = UNSET
    parking_time_minutes: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        timestamp: Union[Unset, str] = UNSET
        if not isinstance(self.timestamp, Unset):
            timestamp = self.timestamp.isoformat()

        delta_kwh = self.delta_kwh

        absolute_kwh = self.absolute_kwh

        price = self.price

        parking_time_minutes = self.parking_time_minutes

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if timestamp is not UNSET:
            field_dict["timestamp"] = timestamp
        if delta_kwh is not UNSET:
            field_dict["deltaKwh"] = delta_kwh
        if absolute_kwh is not UNSET:
            field_dict["absoluteKwh"] = absolute_kwh
        if price is not UNSET:
            field_dict["price"] = price
        if parking_time_minutes is not UNSET:
            field_dict["parkingTimeMinutes"] = parking_time_minutes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _timestamp = d.pop("timestamp", UNSET)
        timestamp: Union[Unset, datetime.datetime]
        if isinstance(_timestamp, Unset) or _timestamp is None:
            timestamp = UNSET
        else:
            timestamp = isoparse(_timestamp)

        delta_kwh = d.pop("deltaKwh", UNSET)

        absolute_kwh = d.pop("absoluteKwh", UNSET)

        price = d.pop("price", UNSET)

        parking_time_minutes = d.pop("parkingTimeMinutes", UNSET)

        charging_period_dto = cls(
            timestamp=timestamp,
            delta_kwh=delta_kwh,
            absolute_kwh=absolute_kwh,
            price=price,
            parking_time_minutes=parking_time_minutes,
        )

        charging_period_dto.additional_properties = d
        return charging_period_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="AuthorizationChargerContextDto")


@_attrs_define
class AuthorizationChargerContextDto:
    """
    Attributes:
        allow_any_token (Union[Unset, bool]):
        is_roaming (Union[Unset, bool]):
        charge_point_ou_code (Union[Unset, str]):
        reimburse_uid (Union[Unset, str]):
        reimburse_ou (Union[Unset, str]):
        has_reimbursement (Union[Unset, bool]):
    """

    allow_any_token: Union[Unset, bool] = UNSET
    is_roaming: Union[Unset, bool] = UNSET
    charge_point_ou_code: Union[Unset, str] = UNSET
    reimburse_uid: Union[Unset, str] = UNSET
    reimburse_ou: Union[Unset, str] = UNSET
    has_reimbursement: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        allow_any_token = self.allow_any_token

        is_roaming = self.is_roaming

        charge_point_ou_code = self.charge_point_ou_code

        reimburse_uid = self.reimburse_uid

        reimburse_ou = self.reimburse_ou

        has_reimbursement = self.has_reimbursement

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if allow_any_token is not UNSET:
            field_dict["allowAnyToken"] = allow_any_token
        if is_roaming is not UNSET:
            field_dict["isRoaming"] = is_roaming
        if charge_point_ou_code is not UNSET:
            field_dict["chargePointOUCode"] = charge_point_ou_code
        if reimburse_uid is not UNSET:
            field_dict["reimburseUid"] = reimburse_uid
        if reimburse_ou is not UNSET:
            field_dict["reimburseOu"] = reimburse_ou
        if has_reimbursement is not UNSET:
            field_dict["hasReimbursement"] = has_reimbursement

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        allow_any_token = d.pop("allowAnyToken", UNSET)

        is_roaming = d.pop("isRoaming", UNSET)

        charge_point_ou_code = d.pop("chargePointOUCode", UNSET)

        reimburse_uid = d.pop("reimburseUid", UNSET)

        reimburse_ou = d.pop("reimburseOu", UNSET)

        has_reimbursement = d.pop("hasReimbursement", UNSET)

        authorization_charger_context_dto = cls(
            allow_any_token=allow_any_token,
            is_roaming=is_roaming,
            charge_point_ou_code=charge_point_ou_code,
            reimburse_uid=reimburse_uid,
            reimburse_ou=reimburse_ou,
            has_reimbursement=has_reimbursement,
        )

        authorization_charger_context_dto.additional_properties = d
        return authorization_charger_context_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.string_segment import StringSegment


T = TypeVar("T", bound="EntityTagHeaderValue")


@_attrs_define
class EntityTagHeaderValue:
    """
    Attributes:
        tag (Union[Unset, StringSegment]):
        is_weak (Union[Unset, bool]):
    """

    tag: Union[Unset, "StringSegment"] = UNSET
    is_weak: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        tag: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tag, Unset):
            tag = self.tag.to_dict()

        is_weak = self.is_weak

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if tag is not UNSET:
            field_dict["tag"] = tag
        if is_weak is not UNSET:
            field_dict["isWeak"] = is_weak

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.string_segment import StringSegment

        d = src_dict.copy()
        _tag = d.pop("tag", UNSET)
        tag: Union[Unset, StringSegment]
        if isinstance(_tag, Unset) or _tag is None:
            tag = UNSET
        else:
            tag = StringSegment.from_dict(_tag)

        is_weak = d.pop("isWeak", UNSET)

        entity_tag_header_value = cls(
            tag=tag,
            is_weak=is_weak,
        )

        entity_tag_header_value.additional_properties = d
        return entity_tag_header_value

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class CsChargingProfilesChargingProfileKind(str, Enum):
    ABSOLUTE = "Absolute"
    RECURRING = "Recurring"
    RELATIVE = "Relative"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.send_local_list_request_update_type import SendLocalListRequestUpdateType
from typing import Union

if TYPE_CHECKING:
    from ..models.authorization_data import AuthorizationData


T = TypeVar("T", bound="SendLocalListRequest")


@_attrs_define
class SendLocalListRequest:
    """
    Attributes:
        list_version (int):
        update_type (SendLocalListRequestUpdateType):  Default: SendLocalListRequestUpdateType.DIFFERENTIAL.
        local_authorization_list (Union[Unset, List['AuthorizationData']]):
    """

    list_version: int
    update_type: SendLocalListRequestUpdateType = (
        SendLocalListRequestUpdateType.DIFFERENTIAL
    )
    local_authorization_list: Union[Unset, List["AuthorizationData"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        list_version = self.list_version

        update_type = self.update_type.value

        local_authorization_list: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.local_authorization_list, Unset):
            local_authorization_list = []
            for local_authorization_list_item_data in self.local_authorization_list:
                local_authorization_list_item = (
                    local_authorization_list_item_data.to_dict()
                )
                local_authorization_list.append(local_authorization_list_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "listVersion": list_version,
                "updateType": update_type,
            }
        )
        if local_authorization_list is not UNSET:
            field_dict["localAuthorizationList"] = local_authorization_list

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.authorization_data import AuthorizationData

        d = src_dict.copy()
        list_version = d.pop("listVersion")

        update_type = SendLocalListRequestUpdateType(d.pop("updateType"))

        local_authorization_list = []
        _local_authorization_list = d.pop("localAuthorizationList", UNSET)
        for local_authorization_list_item_data in _local_authorization_list or []:
            local_authorization_list_item = AuthorizationData.from_dict(
                local_authorization_list_item_data
            )

            local_authorization_list.append(local_authorization_list_item)

        send_local_list_request = cls(
            list_version=list_version,
            update_type=update_type,
            local_authorization_list=local_authorization_list,
        )

        send_local_list_request.additional_properties = d
        return send_local_list_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...types import Unset
import datetime
from ...models.longship_error import LongshipError
from ...models.charge_point_authorize_get_dto import ChargePointAuthorizeGetDto


def _get_kwargs(
    id: str,
    *,
    from_: Union[Unset, datetime.datetime] = UNSET,
    to: Union[Unset, datetime.datetime] = UNSET,
    descending: Union[Unset, bool] = UNSET,
    status: Union[Unset, str] = UNSET,
    search: Union[Unset, str] = UNSET,
    skip: Union[Unset, int] = UNSET,
    take: Union[Unset, int] = UNSET,
    idtag: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    json_from_: Union[Unset, str] = UNSET
    if not isinstance(from_, Unset):
        json_from_ = from_.isoformat()
    params["from"] = json_from_

    json_to: Union[Unset, str] = UNSET
    if not isinstance(to, Unset):
        json_to = to.isoformat()
    params["to"] = json_to

    params["descending"] = descending

    params["status"] = status

    params["search"] = search

    params["skip"] = skip

    params["take"] = take

    params["idtag"] = idtag

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/v1/chargepoints/{id}/authorization".format(
            id=id,
        ),
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[List["ChargePointAuthorizeGetDto"], LongshipError]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = []
        _response_200 = response.json()
        for (
            componentsschemascharge_point_authorize_get_dto_array_item_data
        ) in _response_200:
            componentsschemascharge_point_authorize_get_dto_array_item = (
                ChargePointAuthorizeGetDto.from_dict(
                    componentsschemascharge_point_authorize_get_dto_array_item_data
                )
            )

            response_200.append(
                componentsschemascharge_point_authorize_get_dto_array_item
            )

        return response_200
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = LongshipError.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = LongshipError.from_dict(response.json())

        return response_500
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = LongshipError.from_dict(response.json())

        return response_404
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = LongshipError.from_dict(response.json())

        return response_403
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[List["ChargePointAuthorizeGetDto"], LongshipError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    from_: Union[Unset, datetime.datetime] = UNSET,
    to: Union[Unset, datetime.datetime] = UNSET,
    descending: Union[Unset, bool] = UNSET,
    status: Union[Unset, str] = UNSET,
    search: Union[Unset, str] = UNSET,
    skip: Union[Unset, int] = UNSET,
    take: Union[Unset, int] = UNSET,
    idtag: Union[Unset, str] = UNSET,
) -> Response[Union[List["ChargePointAuthorizeGetDto"], LongshipError]]:
    """Get a list of chargepointauthorizations.

     Get a paged list of chargepointauthorizations, taken the filters into account.

    Args:
        id (str):
        from_ (Union[Unset, datetime.datetime]):
        to (Union[Unset, datetime.datetime]):
        descending (Union[Unset, bool]):
        status (Union[Unset, str]):
        search (Union[Unset, str]):
        skip (Union[Unset, int]):
        take (Union[Unset, int]):
        idtag (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[List['ChargePointAuthorizeGetDto'], LongshipError]]
    """

    kwargs = _get_kwargs(
        id=id,
        from_=from_,
        to=to,
        descending=descending,
        status=status,
        search=search,
        skip=skip,
        take=take,
        idtag=idtag,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    from_: Union[Unset, datetime.datetime] = UNSET,
    to: Union[Unset, datetime.datetime] = UNSET,
    descending: Union[Unset, bool] = UNSET,
    status: Union[Unset, str] = UNSET,
    search: Union[Unset, str] = UNSET,
    skip: Union[Unset, int] = UNSET,
    take: Union[Unset, int] = UNSET,
    idtag: Union[Unset, str] = UNSET,
) -> Optional[Union[List["ChargePointAuthorizeGetDto"], LongshipError]]:
    """Get a list of chargepointauthorizations.

     Get a paged list of chargepointauthorizations, taken the filters into account.

    Args:
        id (str):
        from_ (Union[Unset, datetime.datetime]):
        to (Union[Unset, datetime.datetime]):
        descending (Union[Unset, bool]):
        status (Union[Unset, str]):
        search (Union[Unset, str]):
        skip (Union[Unset, int]):
        take (Union[Unset, int]):
        idtag (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[List['ChargePointAuthorizeGetDto'], LongshipError]
    """

    return sync_detailed(
        id=id,
        client=client,
        from_=from_,
        to=to,
        descending=descending,
        status=status,
        search=search,
        skip=skip,
        take=take,
        idtag=idtag,
    ).parsed


async def asyncio_detailed(
    id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    from_: Union[Unset, datetime.datetime] = UNSET,
    to: Union[Unset, datetime.datetime] = UNSET,
    descending: Union[Unset, bool] = UNSET,
    status: Union[Unset, str] = UNSET,
    search: Union[Unset, str] = UNSET,
    skip: Union[Unset, int] = UNSET,
    take: Union[Unset, int] = UNSET,
    idtag: Union[Unset, str] = UNSET,
) -> Response[Union[List["ChargePointAuthorizeGetDto"], LongshipError]]:
    """Get a list of chargepointauthorizations.

     Get a paged list of chargepointauthorizations, taken the filters into account.

    Args:
        id (str):
        from_ (Union[Unset, datetime.datetime]):
        to (Union[Unset, datetime.datetime]):
        descending (Union[Unset, bool]):
        status (Union[Unset, str]):
        search (Union[Unset, str]):
        skip (Union[Unset, int]):
        take (Union[Unset, int]):
        idtag (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[List['ChargePointAuthorizeGetDto'], LongshipError]]
    """

    kwargs = _get_kwargs(
        id=id,
        from_=from_,
        to=to,
        descending=descending,
        status=status,
        search=search,
        skip=skip,
        take=take,
        idtag=idtag,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    from_: Union[Unset, datetime.datetime] = UNSET,
    to: Union[Unset, datetime.datetime] = UNSET,
    descending: Union[Unset, bool] = UNSET,
    status: Union[Unset, str] = UNSET,
    search: Union[Unset, str] = UNSET,
    skip: Union[Unset, int] = UNSET,
    take: Union[Unset, int] = UNSET,
    idtag: Union[Unset, str] = UNSET,
) -> Optional[Union[List["ChargePointAuthorizeGetDto"], LongshipError]]:
    """Get a list of chargepointauthorizations.

     Get a paged list of chargepointauthorizations, taken the filters into account.

    Args:
        id (str):
        from_ (Union[Unset, datetime.datetime]):
        to (Union[Unset, datetime.datetime]):
        descending (Union[Unset, bool]):
        status (Union[Unset, str]):
        search (Union[Unset, str]):
        skip (Union[Unset, int]):
        take (Union[Unset, int]):
        idtag (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[List['ChargePointAuthorizeGetDto'], LongshipError]
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            from_=from_,
            to=to,
            descending=descending,
            status=status,
            search=search,
            skip=skip,
            take=take,
            idtag=idtag,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..models.session_thresholds_dto_thresholds_hit_item import (
    SessionThresholdsDtoThresholdsHitItem,
)

if TYPE_CHECKING:
    from ..models.session_threshold_value_dto_decimal import (
        SessionThresholdValueDtoDecimal,
    )
    from ..models.session_threshold_check_dto import SessionThresholdCheckDto
    from ..models.session_threshold_value_dto_int_32 import (
        SessionThresholdValueDtoInt32,
    )


T = TypeVar("T", bound="SessionThresholdsDto")


@_attrs_define
class SessionThresholdsDto:
    """
    Attributes:
        thresholds_hit (Union[Unset, List[SessionThresholdsDtoThresholdsHitItem]]):
        min_kwh_ac (Union[Unset, SessionThresholdValueDtoDecimal]):
        max_kwh_ac (Union[Unset, SessionThresholdValueDtoDecimal]):
        min_kwh_dc (Union[Unset, SessionThresholdValueDtoDecimal]):
        max_kwh_dc (Union[Unset, SessionThresholdValueDtoDecimal]):
        min_duration_in_minutes_ac (Union[Unset, SessionThresholdValueDtoInt32]):
        min_duration_in_minutes_dc (Union[Unset, SessionThresholdValueDtoInt32]):
        max_session_age_in_days (Union[Unset, SessionThresholdValueDtoInt32]):
        check_charging_speed (Union[Unset, SessionThresholdCheckDto]):
        check_session_in_future (Union[Unset, SessionThresholdCheckDto]):
    """

    thresholds_hit: Union[Unset, List[SessionThresholdsDtoThresholdsHitItem]] = UNSET
    min_kwh_ac: Union[Unset, "SessionThresholdValueDtoDecimal"] = UNSET
    max_kwh_ac: Union[Unset, "SessionThresholdValueDtoDecimal"] = UNSET
    min_kwh_dc: Union[Unset, "SessionThresholdValueDtoDecimal"] = UNSET
    max_kwh_dc: Union[Unset, "SessionThresholdValueDtoDecimal"] = UNSET
    min_duration_in_minutes_ac: Union[Unset, "SessionThresholdValueDtoInt32"] = UNSET
    min_duration_in_minutes_dc: Union[Unset, "SessionThresholdValueDtoInt32"] = UNSET
    max_session_age_in_days: Union[Unset, "SessionThresholdValueDtoInt32"] = UNSET
    check_charging_speed: Union[Unset, "SessionThresholdCheckDto"] = UNSET
    check_session_in_future: Union[Unset, "SessionThresholdCheckDto"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        thresholds_hit: Union[Unset, List[str]] = UNSET
        if not isinstance(self.thresholds_hit, Unset):
            thresholds_hit = []
            for thresholds_hit_item_data in self.thresholds_hit:
                thresholds_hit_item = thresholds_hit_item_data.value
                thresholds_hit.append(thresholds_hit_item)

        min_kwh_ac: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.min_kwh_ac, Unset):
            min_kwh_ac = self.min_kwh_ac.to_dict()

        max_kwh_ac: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.max_kwh_ac, Unset):
            max_kwh_ac = self.max_kwh_ac.to_dict()

        min_kwh_dc: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.min_kwh_dc, Unset):
            min_kwh_dc = self.min_kwh_dc.to_dict()

        max_kwh_dc: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.max_kwh_dc, Unset):
            max_kwh_dc = self.max_kwh_dc.to_dict()

        min_duration_in_minutes_ac: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.min_duration_in_minutes_ac, Unset):
            min_duration_in_minutes_ac = self.min_duration_in_minutes_ac.to_dict()

        min_duration_in_minutes_dc: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.min_duration_in_minutes_dc, Unset):
            min_duration_in_minutes_dc = self.min_duration_in_minutes_dc.to_dict()

        max_session_age_in_days: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.max_session_age_in_days, Unset):
            max_session_age_in_days = self.max_session_age_in_days.to_dict()

        check_charging_speed: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.check_charging_speed, Unset):
            check_charging_speed = self.check_charging_speed.to_dict()

        check_session_in_future: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.check_session_in_future, Unset):
            check_session_in_future = self.check_session_in_future.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if thresholds_hit is not UNSET:
            field_dict["thresholdsHit"] = thresholds_hit
        if min_kwh_ac is not UNSET:
            field_dict["minKwhAc"] = min_kwh_ac
        if max_kwh_ac is not UNSET:
            field_dict["maxKwhAc"] = max_kwh_ac
        if min_kwh_dc is not UNSET:
            field_dict["minKwhDc"] = min_kwh_dc
        if max_kwh_dc is not UNSET:
            field_dict["maxKwhDc"] = max_kwh_dc
        if min_duration_in_minutes_ac is not UNSET:
            field_dict["minDurationInMinutesAc"] = min_duration_in_minutes_ac
        if min_duration_in_minutes_dc is not UNSET:
            field_dict["minDurationInMinutesDc"] = min_duration_in_minutes_dc
        if max_session_age_in_days is not UNSET:
            field_dict["maxSessionAgeInDays"] = max_session_age_in_days
        if check_charging_speed is not UNSET:
            field_dict["checkChargingSpeed"] = check_charging_speed
        if check_session_in_future is not UNSET:
            field_dict["checkSessionInFuture"] = check_session_in_future

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.session_threshold_value_dto_decimal import (
            SessionThresholdValueDtoDecimal,
        )
        from ..models.session_threshold_check_dto import SessionThresholdCheckDto
        from ..models.session_threshold_value_dto_int_32 import (
            SessionThresholdValueDtoInt32,
        )

        d = src_dict.copy()
        thresholds_hit = []
        _thresholds_hit = d.pop("thresholdsHit", UNSET)
        for thresholds_hit_item_data in _thresholds_hit or []:
            thresholds_hit_item = SessionThresholdsDtoThresholdsHitItem(
                thresholds_hit_item_data
            )

            thresholds_hit.append(thresholds_hit_item)

        _min_kwh_ac = d.pop("minKwhAc", UNSET)
        min_kwh_ac: Union[Unset, SessionThresholdValueDtoDecimal]
        if isinstance(_min_kwh_ac, Unset) or _min_kwh_ac is None:
            min_kwh_ac = UNSET
        else:
            min_kwh_ac = SessionThresholdValueDtoDecimal.from_dict(_min_kwh_ac)

        _max_kwh_ac = d.pop("maxKwhAc", UNSET)
        max_kwh_ac: Union[Unset, SessionThresholdValueDtoDecimal]
        if isinstance(_max_kwh_ac, Unset) or _max_kwh_ac is None:
            max_kwh_ac = UNSET
        else:
            max_kwh_ac = SessionThresholdValueDtoDecimal.from_dict(_max_kwh_ac)

        _min_kwh_dc = d.pop("minKwhDc", UNSET)
        min_kwh_dc: Union[Unset, SessionThresholdValueDtoDecimal]
        if isinstance(_min_kwh_dc, Unset) or _min_kwh_dc is None:
            min_kwh_dc = UNSET
        else:
            min_kwh_dc = SessionThresholdValueDtoDecimal.from_dict(_min_kwh_dc)

        _max_kwh_dc = d.pop("maxKwhDc", UNSET)
        max_kwh_dc: Union[Unset, SessionThresholdValueDtoDecimal]
        if isinstance(_max_kwh_dc, Unset) or _max_kwh_dc is None:
            max_kwh_dc = UNSET
        else:
            max_kwh_dc = SessionThresholdValueDtoDecimal.from_dict(_max_kwh_dc)

        _min_duration_in_minutes_ac = d.pop("minDurationInMinutesAc", UNSET)
        min_duration_in_minutes_ac: Union[Unset, SessionThresholdValueDtoInt32]
        if isinstance(_min_duration_in_minutes_ac, Unset) or _min_duration_in_minutes_ac is None:
            min_duration_in_minutes_ac = UNSET
        else:
            min_duration_in_minutes_ac = SessionThresholdValueDtoInt32.from_dict(
                _min_duration_in_minutes_ac
            )

        _min_duration_in_minutes_dc = d.pop("minDurationInMinutesDc", UNSET)
        min_duration_in_minutes_dc: Union[Unset, SessionThresholdValueDtoInt32]
        if isinstance(_min_duration_in_minutes_dc, Unset) or _min_duration_in_minutes_dc is None:
            min_duration_in_minutes_dc = UNSET
        else:
            min_duration_in_minutes_dc = SessionThresholdValueDtoInt32.from_dict(
                _min_duration_in_minutes_dc
            )

        _max_session_age_in_days = d.pop("maxSessionAgeInDays", UNSET)
        max_session_age_in_days: Union[Unset, SessionThresholdValueDtoInt32]
        if isinstance(_max_session_age_in_days, Unset) or _max_session_age_in_days is None:
            max_session_age_in_days = UNSET
        else:
            max_session_age_in_days = SessionThresholdValueDtoInt32.from_dict(
                _max_session_age_in_days
            )

        _check_charging_speed = d.pop("checkChargingSpeed", UNSET)
        check_charging_speed: Union[Unset, SessionThresholdCheckDto]
        if isinstance(_check_charging_speed, Unset) or _check_charging_speed is None:
            check_charging_speed = UNSET
        else:
            check_charging_speed = SessionThresholdCheckDto.from_dict(
                _check_charging_speed
            )

        _check_session_in_future = d.pop("checkSessionInFuture", UNSET)
        check_session_in_future: Union[Unset, SessionThresholdCheckDto]
        if isinstance(_check_session_in_future, Unset) or _check_session_in_future is None:
            check_session_in_future = UNSET
        else:
            check_session_in_future = SessionThresholdCheckDto.from_dict(
                _check_session_in_future
            )

        session_thresholds_dto = cls(
            thresholds_hit=thresholds_hit,
            min_kwh_ac=min_kwh_ac,
            max_kwh_ac=max_kwh_ac,
            min_kwh_dc=min_kwh_dc,
            max_kwh_dc=max_kwh_dc,
            min_duration_in_minutes_ac=min_duration_in_minutes_ac,
            min_duration_in_minutes_dc=min_duration_in_minutes_dc,
            max_session_age_in_days=max_session_age_in_days,
            check_charging_speed=check_charging_speed,
            check_session_in_future=check_session_in_future,
        )

        session_thresholds_dto.additional_properties = d
        return session_thresholds_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
